@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/CParse/Attic/cscanner.c,v 1.1.2.9 2002/01/31 14:57:06 beazley Exp $";
+char cvsroot_cscanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/cscanner.c,v 1.8 2004/02/20 22:51:47 cheetah Exp $";
 
 #include "cparse.h"
 #include "parser.h"
@@ -33,40 +33,29 @@ InFile  *in_head;
 DOHFile *LEX_in = 0;
 static DOHString     *header = 0;
 static DOHString     *comment = 0;
-DOHString     *scanner_ccode = 0;            /* String containing C code */
+DOHString            *scanner_ccode = 0;            /* String containing C code */
 static char          *yybuffer = 0;
 
 static char    yytext[YYBSIZE];
 static int     yylen = 0;
-int            line_number = 1;
-char          *input_file;
-int            start_line = 0;
+int            cparse_line = 1;
+char          *cparse_file;
+int            cparse_start_line = 0;
 static  int    comment_start;
 static  int    scan_init  = 0;
 static  int    num_brace = 0;
 static  int    last_brace = 0;
-extern  int    Error;
 static  int    last_id = 0;
+static  int    rename_active = 0;
+        int    cparse_cplusplus;
 
+/* -----------------------------------------------------------------------------
+ * Swig_cparse_cplusplus()
+ * ----------------------------------------------------------------------------- */
 
-static int       fatal_errors = 0;         
-
-/* Handle an error */
-void cparse_error(String *file, int line, char *fmt, ...) {
-  va_list ap;
-  va_start(ap,fmt);
-  if (line > 0) {
-    Printf(stderr,"%s:%d. ", file, line);
-  } else {
-    Printf(stderr,"%s:EOF ",file);
-  }
-  vPrintf(stderr,fmt,ap);
-  va_end(ap);
-  fatal_errors++;
-}
-
-int CParse_errors(void) {
-  return fatal_errors;
+void
+Swig_cparse_cplusplus(int v) {
+  cparse_cplusplus = v;
 }
 
 /* ----------------------------------------------------------------------
@@ -94,20 +83,20 @@ scanner_locator(String *loc) {
   if (c == '@') {
     /* Empty locator.  We pop the last location off */
     if (locs) {
-      input_file = locs->filename;
-      line_number = locs->line_number;
+      cparse_file = locs->filename;
+      cparse_line = locs->line_number;
       l = locs->next;
       free(locs);
       locs = l;
     }
-    /*    Printf(stderr,"location: %s:%d\n",input_file,line_number);*/
+    /*    Printf(stderr,"location: %s:%d\n",cparse_file,cparse_line);*/
     return;
   }
 
   /* We're going to push a new location */
   l = (Locator *) malloc(sizeof(Locator));
-  l->filename = input_file;
-  l->line_number = line_number;
+  l->filename = cparse_file;
+  l->line_number = cparse_line;
   l->next = locs;
   locs = l;
   
@@ -121,17 +110,17 @@ scanner_locator(String *loc) {
       Putc(c,fn);
     }
 
-    input_file = Swig_copy_string(Char(fn));
+    cparse_file = Swig_copy_string(Char(fn));
     Clear(fn);
   
-    line_number = 1;
+    cparse_line = 1;
     /* Get the line number */
     while ((c = Getc(loc)) != EOF) {
       if ((c == '@') || (c == ',')) break;
       Putc(c,fn);
     }
 
-    line_number = atoi(Char(fn));
+    cparse_line = atoi(Char(fn));
     Clear(fn);
 
     /* Get the rest of it */
@@ -139,7 +128,7 @@ scanner_locator(String *loc) {
       if (c == '@') break;
       Putc(c,fn);
     }
-    /*  Printf(stderr,"location: %s:%d\n",input_file,line_number); */
+    /*  Printf(stderr,"location: %s:%d\n",cparse_file,cparse_line); */
     Delete(fn);
   }
 }
@@ -168,13 +157,13 @@ void scanner_file(DOHFile *f) {
 
   in = (InFile *) malloc(sizeof(InFile));
   in->f = f;
-  in->in_file = input_file;
+  in->in_file = cparse_file;
   in->line_number = 1;
   if (!in_head) in->prev = 0;
   else in->prev = in_head;
   in_head = in;
   LEX_in = f;
-  line_number = 1;
+  cparse_line = 1;
 }
 
 /**************************************************************
@@ -190,8 +179,8 @@ void scanner_close() {
   p = in_head->prev;
   if (p != 0) {
     LEX_in = p->f;
-    line_number = p->line_number;
-    input_file = p->in_file;
+    cparse_line = p->line_number;
+    cparse_file = p->in_file;
   } else {
     LEX_in = 0;
   }
@@ -226,7 +215,7 @@ char nextchar() {
     yytext[yylen] = c;
     yylen++;
     if (c == '\n') {
-      line_number++;
+      cparse_line++;
     }
     return(c);
 }
@@ -238,7 +227,7 @@ void retract(int n) {
     if (yylen >= 0) {
       Ungetc(yytext[yylen],LEX_in);
       if (yytext[yylen] == '\n') {
-	line_number--;
+	cparse_line--;
       }
     }
   }
@@ -260,18 +249,18 @@ void start_inline(char *text, int line) {
   InFile *in;
 
   /* Save current state */
-  in_head->line_number = line_number;
-  in_head->in_file = input_file;
+  in_head->line_number = cparse_line;
+  in_head->in_file = cparse_file;
 
   in = (InFile *) malloc(sizeof(InFile));
   in->f = NewString(text);
   Seek(in->f,0,SEEK_SET);
-  in->in_file = Swig_copy_string(input_file);
+  in->in_file = Swig_copy_string(cparse_file);
   in->line_number = line;
   in->prev = in_head;
   in_head = in;
   LEX_in = in->f;
-  line_number = line;
+  cparse_line = line;
 }
 
 /**************************************************************
@@ -280,9 +269,10 @@ void start_inline(char *text, int line) {
  * Inserts a comment into a documentation entry.
  **************************************************************/
 
+/*
 void yycomment(char *a, int b, int c) {
 }
-
+*/
 
 /* -----------------------------------------------------------------------------
  * skip_balanced()
@@ -297,70 +287,63 @@ skip_balanced(int startchar, int endchar) {
     int  num_levels = 1;
     int  state = 0;
     char temp[2] = {0,0};
+    int  start_line = cparse_line;
 
     Clear(scanner_ccode);
     Putc(startchar,scanner_ccode);
     temp[0] = (char) startchar;
     while (num_levels > 0) {
-	if ((c = nextchar()) == 0) {
-	  cparse_error(input_file, line_number, "Missing '%c'. Reached end of input.\n", endchar);
+      c = nextchar();
+      if (c == 0) {
+	  Swig_error(cparse_file, start_line, "Missing '%c'. Reached end of input.\n", endchar);
 	  return;
-	}
-	Putc(c,scanner_ccode);
-	switch(state) {
-	case 0:
-	    if (c == startchar) num_levels++;
-	    else if (c == endchar) num_levels--;
+      }
+      Putc(c,scanner_ccode);
+      switch(state) {
+      case 0:
+	if (c == startchar) num_levels++;
+	else if (c == endchar) num_levels--;
 	    else if (c == '/') state = 10;
-	    else if (c == '\"') state = 20;
-	    else if (c == '\'') state = 30;
-	    break;
-	case 10:
-	    if (c == '/') state = 11;
-	    else if (c == '*') state = 12;
-	    else state = 0;
-	    break;
-	case 11:
-	    if (c == '\n') state = 0;
-	    else state = 11;
-	    break;
-	case 12:
-	    if (c == '*') state = 13;
-	    break;
-	case 13:
-	    if (c == '*') state = 13;
-	    else if (c == '/') state = 0;
-	    else state = 12;
-	    break;
-	case 20:
-	    if (c == '\"') state = 0;
-	    else if (c == '\\') state = 21;
-	    break;
-	case 21:
-	    state = 20;
-	    break;
-	case 30:
-	    if (c == '\'') state = 0;
-	    else if (c == '\\') state = 31;
-	    break;
-	case 31:
-	    state = 30;
-	    break;
-	default:
-	    break;
-	}
-	yylen = 0;
+	else if (c == '\"') state = 20;
+	else if (c == '\'') state = 30;
+	break;
+      case 10:
+	if (c == '/') state = 11;
+	else if (c == '*') state = 12;
+	else state = 0;
+	break;
+      case 11:
+	if (c == '\n') state = 0;
+	else state = 11;
+	break;
+      case 12:
+	if (c == '*') state = 13;
+	break;
+      case 13:
+	if (c == '*') state = 13;
+	else if (c == '/') state = 0;
+	else state = 12;
+	break;
+      case 20:
+	if (c == '\"') state = 0;
+	else if (c == '\\') state = 21;
+	break;
+      case 21:
+	state = 20;
+	break;
+      case 30:
+	if (c == '\'') state = 0;
+	else if (c == '\\') state = 31;
+	break;
+      case 31:
+	state = 30;
+	break;
+      default:
+	break;
+      }
+      yylen = 0;
     }
-    
-    /* Sick hack alert.  We look for type-escapes and replace them here */
-    /* might not need.
-    if (strchr(Char(scanner_ccode),'`')) {
-      String *ns = Copy(scanner_ccode);
-      Clear(scanner_ccode);
-      Printf(scanner_ccode,"%(typecode)s",ns);
-      Delete(ns);
-    }
-    */
+    if (endchar == '}') num_brace--;
     return;
 }
 
@@ -381,7 +364,7 @@ void skip_decl(void) {
   int  done = 0;
   while (!done) {
     if ((c = nextchar()) == 0) {
-      cparse_error(input_file,line_number,"Missing semicolon. Reached end of input.\n");
+      Swig_error(cparse_file,cparse_line,"Missing semicolon. Reached end of input.\n");
       return;
     }
     if (c == '{') {
@@ -395,7 +378,7 @@ void skip_decl(void) {
   if (!done) {
     while (num_brace > last_brace) {
       if ((c = nextchar()) == 0) {
-	cparse_error(input_file,line_number,"Missing '}'. Reached end of input.\n");
+	Swig_error(cparse_file,cparse_line,"Missing '}'. Reached end of input.\n");
 	return;
       }
       if (c == '{') num_brace++;
@@ -409,7 +392,7 @@ void skip_decl(void) {
 static void get_escape() {
   int result = 0;
   int state = 0;
-  char  c;
+  int c;
 
   while(1) {
     c = nextchar();
@@ -464,8 +447,9 @@ static void get_escape() {
 	yylen--;
 	return;
       }
-      if (c == '0') {
+      if (isdigit(c)) {
 	state = 10;
+	result = (c-'0');
       }
       else if (c == 'x') {
 	state = 20;
@@ -512,7 +496,7 @@ static void get_escape() {
 int yylook(void) {
 
     int      state;
-    char     c = 0;
+    int      c = 0;
 
     state = 0;
     yylen = 0;
@@ -529,11 +513,11 @@ int yylook(void) {
 	  if (c == '\n') {
 	    state = 0;
 	    yylen = 0;
-	    last_id = 0;
+	    /*	    last_id = 0;*/
 	  } else if (isspace(c) || (c=='\\')) {
 	    state = 0;
 	    yylen = 0;
-	    last_id = 0;
+	    /*	    last_id = 0; */
 	  }
 
 	  else if ((isalpha(c)) || (c == '_')) state = 7;
@@ -549,8 +533,7 @@ int yylook(void) {
 	  else if (c == '}') {
 	    num_brace--;
 	    if (num_brace < 0) {
-	      Printf(stderr,"%s:%d. Error. Extraneous '}' (Ignored)\n",
-		      input_file, line_number);
+	      Swig_error(cparse_file, cparse_line, "Syntax error. Extraneous '}'\n");
 	      state = 0;
 	      num_brace = 0;
 	    } else {
@@ -574,7 +557,9 @@ int yylook(void) {
 	  else if (c == '^') return (XOR);
           else if (c == '<') state = 60;
 	  else if (c == '>') state = 61;
-	  else if (c == '~') return (NOT);
+	  else if (c == '~') {
+	    return (NOT);
+	  }
           else if (c == '!') return (LNOT);
 	  else if (c == '\\') {
 	    state = 99;
@@ -604,11 +589,11 @@ int yylook(void) {
 	case 1:  /*  Comment block */
 	  if ((c = nextchar()) == 0) return(0);
 	  if (c == '/') {
-	    comment_start = line_number;
+	    comment_start = cparse_line;
 	    Clear(comment);
 	    state = 10;        /* C++ style comment */
 	  } else if (c == '*') {
-	    comment_start = line_number;
+	    comment_start = cparse_line;
 	    Clear(comment);
 	    state = 12;   /* C style comment */
 	  } else {
@@ -633,7 +618,7 @@ int yylook(void) {
 	  }
 	case 10:  /* C++ style comment */
 	  if ((c = nextchar()) == 0) {
-	    cparse_error(input_file,-1, "Unterminated comment detected.\n");
+	    Swig_error(cparse_file,-1, "Unterminated comment detected.\n");
 	    return 0;
 	  }
 	  if (c == '\n') {
@@ -651,7 +636,7 @@ int yylook(void) {
 
 	case 12: /* C style comment block */
 	  if ((c = nextchar()) == 0) {
-	    cparse_error(input_file,-1,"Unterminated comment detected.\n");
+	    Swig_error(cparse_file,-1,"Unterminated comment detected.\n");
 	    return 0;
 	  }
 	  if (c == '*') {
@@ -664,7 +649,7 @@ int yylook(void) {
 	  break;
 	case 13: /* Still in C style comment */
 	  if ((c = nextchar()) == 0) {
-	    cparse_error(input_file,-1,"Unterminated comment detected.\n");
+	    Swig_error(cparse_file,-1,"Unterminated comment detected.\n");
 	    return 0;
 	  }
 	  if (c == '*') {
@@ -695,7 +680,7 @@ int yylook(void) {
 
 	case 2: /* Processing a string */
 	  if ((c = nextchar()) == 0) {
-	    cparse_error(input_file,-1, "Unterminated string detected.\n");
+	    Swig_error(cparse_file,-1, "Unterminated string detected.\n");
 	    return 0;
 	  }
 	  if (c == '\"') {
@@ -724,10 +709,10 @@ int yylook(void) {
 	  if (c == '{') {
 	    state = 40;   /* Include block */
 	    Clear(header);
-	    start_line = line_number;
+	    cparse_start_line = cparse_line;
 	  } else if ((isalpha(c)) || (c == '_')) state = 7;
 	  else if (c == '}') {
-	    cparse_error(input_file,line_number, "Misplaced %%}.\n");
+	    Swig_error(cparse_file,cparse_line, "Misplaced %%}.\n");
 	    return 0;
 	  } else {
 	    retract(1);
@@ -737,7 +722,7 @@ int yylook(void) {
 
 	case 40: /* Process an include block */
 	  if ((c = nextchar()) == 0) {
-	    cparse_error(input_file,-1, "Unterminated include block detected.\n");
+	    Swig_error(cparse_file,-1, "Unterminated include block detected.\n");
 	    return 0;
 	  }
 	  yylen = 0;
@@ -750,7 +735,7 @@ int yylook(void) {
 	  break;
 	case 41: /* Still processing include block */
 	  if ((c = nextchar()) == 0) {
-	    cparse_error(input_file,-1, "Unterminated include block detected.\n");
+	    Swig_error(cparse_file,-1, "Unterminated include block detected.\n");
 	    return 0;
 	  }
 	  if (c == '}') {
@@ -774,10 +759,14 @@ int yylook(void) {
 	    return COLON;
 	  }
 	  break;
-	case 51: /* Maybe a ::* or :: */
+	case 51: /* Maybe a ::*, ::~, or :: */
 	  if (( c = nextchar()) == 0) return 0;
 	  if (c == '*') {
 	    return DSTAR;
+	  } else if (c == '~') {
+	    return DCNOT;
+	  } else if (isspace(c)) {
+	    /* Keep scanning ahead.  Might be :: * or :: ~ */
 	  } else {
 	    retract(1);
 	    if (!last_id) {
@@ -787,6 +776,7 @@ int yylook(void) {
 	      return DCOLON;
 	    }
 	  }
+	  break;
 
 	case 60: /* shift operators */
 	  if ((c = nextchar()) == 0) return (0);
@@ -970,7 +960,7 @@ int yylook(void) {
 	    yytext[yylen-1] = 0;
 	    yylval.str = NewString(yytext+1);
 	    if (yylen == 2) {
-	      Printf(stderr,"%s:%d. Empty character constant\n", input_file,line_number);
+	      Swig_error(cparse_file, cparse_line, "Empty character constant\n");
 	    }
 	    return(CHARCONST);
 	  }
@@ -994,11 +984,8 @@ int yylook(void) {
 	  break;
 
 	default:
-	  if (!Error) {
-	    cparse_error(input_file, line_number, "Illegal character '%c'=%d.\n",c,c);
-	  }
+	  Swig_error(cparse_file, cparse_line, "Illegal character '%c'=%d.\n",c,c);
 	  state = 0;
-  	  Error = 1;
 	  return(ILLEGAL);
 	}
     }
@@ -1014,6 +1001,19 @@ void scanner_ignore_typedef() {
   check_typedef = 0;
 }
 
+void scanner_last_id(int x) {
+  /*  printf("Setting last_id = %d\n", x); */
+  last_id = x;
+}
+
+void scanner_clear_rename() {
+  rename_active = 0;
+}
+
+static int next_token = 0;
+void scanner_next_token(int tok) {
+  next_token = tok;
+}
 
 /**************************************************************
  * int yylex()
@@ -1029,20 +1029,29 @@ int yylex(void) {
       scanner_init();
     }
 
+    if (next_token) {
+      l = next_token;
+      next_token = 0;
+      return l;
+    }
+    /*    Printf(stdout,"%d\n", last_id);*/
     l = yylook();
 
-
     if (l == NONID) {
       last_id = 1;
     } else {
       last_id = 0;
     }
+    /*
+    yytext[yylen]= 0;
+    Printf(stdout,"%d  '%s' %d\n", l, yytext, last_id);
+    */
 
     /* We got some sort of non-white space object.  We set the start_line
        variable unless it has already been set */
 
-    if (!start_line) {
-      start_line = line_number;
+    if (!cparse_start_line) {
+      cparse_start_line = cparse_line;
     }
 
     /* Copy the lexene */
@@ -1065,6 +1074,8 @@ int yylex(void) {
       if (l == NUM_LONGLONG) yylval.dtype.type = T_LONGLONG;
       if (l == NUM_ULONGLONG) yylval.dtype.type = T_ULONGLONG;
       yylval.dtype.val = NewString(yytext);
+      yylval.dtype.bitfield = 0;
+      yylval.dtype.throws = 0;
       return(l);
       break;
       
@@ -1115,7 +1126,7 @@ int yylex(void) {
 	  }
 	  /* C++ keywords */
 	  
-	  if (CPlusPlus) {
+	  if (cparse_cplusplus) {
 	    if (strcmp(yytext,"class") == 0) return(CLASS);
 	    if (strcmp(yytext,"private") == 0) return(PRIVATE);
 	    if (strcmp(yytext,"public") == 0) return(PUBLIC);
@@ -1148,9 +1159,15 @@ int yylex(void) {
 	      }
 	      Chop(s);
 	      yylval.str = s;
+	      while(Replaceall(s,"[ ", "["));
 	      if (isconversion) {
+		String *ns = Swig_symbol_string_qualify(s,0);
+		yylval.str = ns;
+	      }
+	      if (isconversion && !rename_active) {
 		char *t = Char(s) + 9;
-		if (!((strcmp(t,"new") == 0) || (strcmp(t,"delete") == 0))) {
+		if (!((strcmp(t,"new") == 0) || (strcmp(t,"delete") == 0) 
+		      || (strcmp(t,"new[]") == 0) || (strcmp(t,"delete[]") == 0))) {
 		  /*		  retract(strlen(t));*/
 		  retract(count);
 		  return COPERATOR;
@@ -1159,23 +1176,29 @@ int yylex(void) {
 	      return(OPERATOR);
 	    }
 	    if (strcmp(yytext,"throw") == 0) return(THROW);
+	    if (strcmp(yytext,"try") == 0) return (yylex());
+	    if (strcmp(yytext,"catch") == 0) return (CATCH);
 	    if (strcmp(yytext,"inline") == 0) return(yylex());
 	    if (strcmp(yytext,"mutable") == 0) return(yylex());
 	    if (strcmp(yytext,"explicit") == 0) return(yylex());
+	    if (strcmp(yytext,"export") == 0) return(yylex());
 	    if (strcmp(yytext,"typename") == 0) return (TYPENAME);
 	    if (strcmp(yytext,"template") == 0) {
-	      yylval.ivalue = line_number;
+	      yylval.ivalue = cparse_line;
 	      return(TEMPLATE);
 	    }
-	    if (strcmp(yytext,"new") == 0) {
-	      return(NEW);
-	    }
 	    if (strcmp(yytext,"delete") == 0) {
-	      return(DELETE);
+	      return(DELETE_KW);
+	    }
+	    if (strcmp(yytext,"using") == 0) {
+	      return(USING);
+	    }
+	    if (strcmp(yytext,"namespace") == 0) {
+	      return(NAMESPACE);
 	    }
 	  } else {
 	    if (strcmp(yytext,"class") == 0) {
-	      Printf(stderr,"%s:%d. Warning: class keyword used, but not in C++ mode.\n",input_file,line_number);
+	      Swig_warning(WARN_PARSE_CLASS_KEYWORD,cparse_file,cparse_line, "class keyword used, but not in C++ mode.\n");
 	    }
 	  }
 	  
@@ -1196,7 +1219,7 @@ int yylex(void) {
 	  /* Misc keywords */
 	  
 	  if (strcmp(yytext,"extern") == 0) return(EXTERN);
-	  if (strcmp(yytext,"const") == 0) return(CONST);
+	  if (strcmp(yytext,"const") == 0) return(CONST_QUAL);
 	  if (strcmp(yytext,"static") == 0) return(STATIC);
 	  if (strcmp(yytext,"struct") == 0) return(STRUCT);
 	  if (strcmp(yytext,"union") == 0) return(UNION);
@@ -1216,17 +1239,22 @@ int yylex(void) {
 	} else {
 	  if (strcmp(yytext,"%module") == 0) return(MODULE);
 	  if (strcmp(yytext,"%insert") == 0) return(INSERT);
-	  if (strcmp(yytext,"%gencode") == 0) return (GENCODE);
 	  if (strcmp(yytext,"%name") == 0) return(NAME);
-	  if (strcmp(yytext,"%rename") == 0) return(RENAME);
-	  if (strcmp(yytext,"%namewarn") == 0) return (NAMEWARN);
+	  if (strcmp(yytext,"%rename") == 0) {
+	    rename_active = 1;
+	    return(RENAME);
+	  }
+	  if (strcmp(yytext,"%namewarn") == 0) {
+	    rename_active = 1;
+	    return (NAMEWARN);
+	  }
 	  if (strcmp(yytext,"%includefile") == 0) return(INCLUDE);
 	  if (strcmp(yytext,"%val") == 0) {
-	    Printf(stderr,"%s:%d %%val directive deprecated (ignored).\n", input_file, line_number);
+	    Swig_warning(WARN_DEPRECATED_VAL, cparse_file, cparse_line, "%%val directive deprecated (ignored).\n");
 	    return (yylex());
 	  }
 	  if (strcmp(yytext,"%out") == 0) {
-	    Printf(stderr,"%s:%d %%out directive deprecated (ignored).\n", input_file, line_number);
+	    Swig_warning(WARN_DEPRECATED_OUT, cparse_file, cparse_line, "%%out directive deprecated (ignored).\n");
 	    return(yylex());
 	  }
 	  if (strcmp(yytext,"%constant") == 0) return(CONSTANT);
@@ -1236,19 +1264,21 @@ int yylex(void) {
 	  }
 	  if (strcmp(yytext,"%native") == 0) return(NATIVE);
 	  if (strcmp(yytext,"%pragma") == 0) return(PRAGMA);
-	  if (strcmp(yytext,"%addmethods") == 0) return(ADDMETHODS);
+	  if (strcmp(yytext,"%extend") == 0) return(EXTEND);
+	  if (strcmp(yytext,"%fragment") == 0) return(FRAGMENT);
 	  if (strcmp(yytext,"%inline") == 0) return(INLINE);
 	  if (strcmp(yytext,"%typemap") == 0) return(TYPEMAP);
 	  if (strcmp(yytext,"%feature") == 0) return(FEATURE);
 	  if (strcmp(yytext,"%except") == 0) return(EXCEPT);
 	  if (strcmp(yytext,"%importfile") == 0) return(IMPORT);
 	  if (strcmp(yytext,"%echo") == 0) return(ECHO);
-	  if (strcmp(yytext,"%new") == 0) return(NEW);
 	  if (strcmp(yytext,"%apply") == 0) return(APPLY);
 	  if (strcmp(yytext,"%clear") == 0) return(CLEAR);
 	  if (strcmp(yytext,"%types") == 0) return(TYPES);
+	  if (strcmp(yytext,"%parms") == 0) return(PARMS);
+	  if (strcmp(yytext,"%varargs") == 0) return(VARARGS);
 	  if (strcmp(yytext,"%template") == 0) return (SWIGTEMPLATE);
-	  if (strcmp(yytext,"%endtemplate") == 0) return (ENDTEMPLATE);
+	  if (strcmp(yytext,"%warn") == 0) return(WARN);
 	}
 	/* Have an unknown identifier, as a last step, we'll do a typedef lookup on it. */
 
