@@ -15,8 +15,9 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Preprocessor/cpp.c,v 1.27.2.1.2.18 2002/01/26 15:57:47 beazley Exp $";
+char cvsroot_cpp_c[] = "$Header: /cvsroot/swig/SWIG/Source/Preprocessor/cpp.c,v 1.53 2004/03/17 08:32:11 mmatus Exp $";
 
+#include "swig.h"
 #include "preprocessor.h"
 #include <ctype.h>
 
@@ -25,40 +26,19 @@ static int       include_all = 0;        /* Follow all includes */
 static int       ignore_missing = 0; 
 static int       import_all = 0;         /* Follow all includes, but as %import statements */
 static int       single_include = 1;     /* Only include each file once */
-static int       silent_errors = 0;
-static int       fatal_errors = 0;         
 static Hash     *included_files = 0;
-
-/* Handle an error */
-
-static void cpp_error(String *file, int line, char *fmt, ...) {
-  va_list ap;
-  if (silent_errors) return;
-  va_start(ap,fmt);
-  if (line > 0) {
-    Printf(stderr,"%s:%d. ", file, line);
-  } else {
-    Printf(stderr,"%s:EOF ",file);
-  }
-  vPrintf(stderr,fmt,ap);
-  va_end(ap);
-  fatal_errors++;
-}
-
-int Preprocessor_errors(void) {
-  return fatal_errors;
-}
+static List     *dependencies = 0;
 
 /* Test a character to see if it starts an identifier */
 static int
-isidentifier(char c) {
+isidentifier(int c) {
   if ((isalpha(c)) || (c == '_') || (c == '$')) return 1;
   else return 0;
 }
 
 /* Test a character to see if it valid in an identifier (after the first letter) */
 static int
-isidchar(char c) {
+isidchar(int c) {
   if ((isalnum(c)) || (c == '_') || (c == '$')) return 1;
   else return 0;
 }
@@ -99,28 +79,40 @@ copy_location(DOH *s1, DOH *s2) {
 
 static String *cpp_include(String_or_char *fn) {
   String *s;
-  if (single_include) {
-    if (Getattr(included_files,fn)) return 0;
-    Setattr(included_files,fn,fn);
-  }
   s = Swig_include(fn);
+  if (s && single_include) {
+    String *file = Getfile(s);
+    if (Getattr(included_files,file)) {
+      Delete(s);
+      return 0;
+    }
+    Setattr(included_files,file,file);
+  }
   if (!s) {
     Seek(fn,0,SEEK_SET);
-    cpp_error(Getfile(fn),Getline(fn),"Unable to find '%s'\n", fn);
     if (ignore_missing) {
-      fatal_errors--;
-    } 
+      Swig_warning(WARN_PP_MISSING_FILE,Getfile(fn),Getline(fn),"Unable to find '%s'\n", fn);
+    } else {
+      Swig_error(Getfile(fn),Getline(fn),"Unable to find '%s'\n", fn);
+    }
   } else {
     Seek(s,0,SEEK_SET);
+    if (!dependencies) {
+      dependencies = NewList();
+    }
+    Append(dependencies, Copy(Swig_last_file()));
   }
   return s;
 }
 
+List *Preprocessor_depend(void) {
+  return dependencies;
+}
+
 /* -----------------------------------------------------------------------------
  * void Preprocessor_cpp_init() - Initialize the preprocessor
  * ----------------------------------------------------------------------------- */
-void Preprocessor_init() {
-  extern void Preprocessor_expr_init(void);
+void Preprocessor_init(void) {
   Hash *s;
   cpp = NewHash();
   s =   NewHash();
@@ -133,12 +125,10 @@ void Preprocessor_init() {
  * ----------------------------------------------------------------------------- */
 void Preprocessor_include_all(int a) {
   include_all = a;
-  if (import_all) import_all = 0;
 }
 
 void Preprocessor_import_all(int a) {
   import_all = a;
-  if (include_all) include_all = 0;
 }
 
 void Preprocessor_ignore_missing(int a) {
@@ -153,25 +143,53 @@ void Preprocessor_ignore_missing(int a) {
  * SWIG macro semantics.
  * ----------------------------------------------------------------------------- */
 
-Hash *Preprocessor_define(String_or_char *str, int swigmacro)
+ 
+String_or_char *Macro_vararg_name(String_or_char *str,
+                                  String_or_char *line)
+{
+  String_or_char *argname, *varargname;
+  char *s, *dots;
+
+  argname = Copy(str);
+  s = Char(argname);
+  dots = strchr(s, '.');
+  if (!dots) return NULL;
+  if (strcmp(dots, "...") != 0) {
+    Swig_error(Getfile(line), Getline(line),
+               "Illegal macro argument name '%s'\n", str);  
+    return NULL;
+  }
+  if (dots == s) {
+      varargname = NewString("__VA_ARGS__");
+  } else {
+    *dots = '\0';
+    varargname = NewStringf(argname);
+  }
+  Delete(argname);
+  return varargname;
+}
+
+Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
 {
   String *macroname = 0, *argstr = 0, *macrovalue = 0, *file = 0, *s = 0;
   Hash   *macro = 0, *symbols = 0, *m1;
   List   *arglist = 0;
   int c, line;
+  int    varargs = 0;
+  String_or_char *str = (String_or_char *) _str;
 
   assert(cpp);
   assert(str);
 
   /* First make sure that string is actually a string */
   if (DohCheck(str)) {
-    s = Copy(str);
+    s = NewString(str);
     copy_location(str,s);
     str = s;
   } else {
     str = NewString((char *) str);
-    Seek(str,0,SEEK_SET);
   }
+  Seek(str,0,SEEK_SET);
   line = Getline(str);
   file = Getfile(str);
 
@@ -190,7 +208,7 @@ Hash *Preprocessor_define(String_or_char *str, int swigmacro)
 	else Putc(c,argstr);
       }
       if (c != ')') {
-	cpp_error(Getfile(str),Getline(str), "Missing \')\' in macro parameters\n");
+	Swig_error(Getfile(str),Getline(str), "Missing \')\' in macro parameters\n");
 	goto macro_error;
       }
       break;
@@ -198,9 +216,18 @@ Hash *Preprocessor_define(String_or_char *str, int swigmacro)
       Putc(c,macroname);
     } else if (isspace(c)) {
       break;
+    } else if (c == '\\') {
+      c = Getc(str);
+      if (c != '\n') {
+	Ungetc(c,str);
+	Ungetc('\\',str);
+	break;
+      }
     } else {
-      cpp_error(Getfile(str),Getline(str),"Illegal character in macro name\n");
-      goto macro_error;
+      /*Swig_error(Getfile(str),Getline(str),"Illegal character in macro name\n");
+	goto macro_error; */
+      Ungetc(c,str);
+      break;
     }
   }
   if (!swigmacro)
@@ -212,24 +239,37 @@ Hash *Preprocessor_define(String_or_char *str, int swigmacro)
 
   /* If there are any macro arguments, convert into a list */
   if (argstr) {
-    DOH *argname;
+    String *argname, *varargname;
     arglist = NewList();
     Seek(argstr,0,SEEK_SET);
     argname = NewString("");
     while ((c = Getc(argstr)) != EOF) {
       if (c == ',') {
-	Append(arglist,argname);
+	varargname = Macro_vararg_name(argname, str);
+	if (varargname) {
+          Swig_error(Getfile(str),Getline(str),"Variable-length macro argument must be last parameter\n");	  
+	} else {
+	  Append(arglist,argname);
+	}
 	Delete(argname);
 	argname = NewString("");
-      } else if (isidchar(c)) {
+      } else if (isidchar(c) || (c == '.')) {
 	Putc(c,argname);
-      } else if (!isspace(c)) {
-	cpp_error(Getfile(str),Getline(str),"Illegal character in macro name\n");
+      } else if (!(isspace(c) || (c == '\\'))) {
+	Swig_error(Getfile(str),Getline(str),"Illegal character in macro argument name\n");
 	goto macro_error;
       }
     }
     if (Len(argname)) {
-      Append(arglist,argname);
+      /* Check for varargs */
+      varargname = Macro_vararg_name(argname, str);
+      if (varargname) {
+	Append(arglist,varargname);
+	Delete(varargname);
+	varargs = 1;
+      } else {
+	Append(arglist,argname);
+      }
       Delete(argname);
     }
   }
@@ -301,14 +341,20 @@ Hash *Preprocessor_define(String_or_char *str, int swigmacro)
   }
   /* Replace '##' with a special token */
   Replace(macrovalue,"\001\001","\002", DOH_REPLACE_ANY);
+  /* Replace '#@' with a special token */
+  Replace(macrovalue,"\001@","\004", DOH_REPLACE_ANY);
+  /* Replace '##@' with a special token */
+  Replace(macrovalue,"\002@","\005", DOH_REPLACE_ANY);  
 
   /* Go create the macro */
   macro = NewHash();
   Setattr(macro,"name", macroname);
-  Delete(macroname);
   if (arglist) {
     Setattr(macro,"args",arglist);
     Delete(arglist);
+    if (varargs) {
+      Setattr(macro,"varargs","1");
+    }
   }
   Setattr(macro,"value",macrovalue);
   Delete(macrovalue);
@@ -319,17 +365,23 @@ Hash *Preprocessor_define(String_or_char *str, int swigmacro)
   }
   symbols = Getattr(cpp,"symbols");
   if ((m1 = Getattr(symbols,macroname))) {
-    if (Cmp(Getattr(m1,"value"),macrovalue))
-      cpp_error(Getfile(str),Getline(str),"Macro '%s' redefined. Previous definition in \'%s\', Line %d\n", macroname, Getfile(m1), Getline(m1));
+    if (Cmp(Getattr(m1,"value"),macrovalue)) {
+      Swig_error(Getfile(str),Getline(str),"Macro '%s' redefined,\n",macroname);    
+      Swig_error(Getfile(m1),Getline(m1),"previous definition of '%s'.\n",macroname);
+      goto macro_error;
+    }
   }
   Setattr(symbols,macroname,macro);
+  
   Delete(str);
   Delete(argstr);
+  Delete(macroname);
   return macro;
 
  macro_error:
   Delete(str);
   Delete(argstr);
+  Delete(macroname);
   return 0;
 }
 
@@ -338,7 +390,7 @@ Hash *Preprocessor_define(String_or_char *str, int swigmacro)
  *
  * Undefines a macro.
  * ----------------------------------------------------------------------------- */
-void Preprocessor_undef(String_or_char *str)
+void Preprocessor_undef(const String_or_char *str)
 {
   Hash *symbols;
   assert(cpp);
@@ -373,7 +425,8 @@ find_args(String *s)
   if (c != '(') {
     /* Not a macro, bail out now! */
     Seek(s,pos, SEEK_SET);
-    return args;
+    Delete(args);
+    return 0;
   }
   c = Getc(s);
   /* Okay.  This appears to be a macro so we will start isolating arguments */
@@ -419,7 +472,7 @@ find_args(String *s)
     c = Getc(s);
   }
  unterm:
-  cpp_error(Getfile(args),Getline(args),"Unterminated macro call.\n");
+  Swig_error(Getfile(args),Getline(args),"Unterminated macro call.\n");
   return args;
 }
 
@@ -448,10 +501,43 @@ get_filename(String *str) {
     while (((c = Getc(str)) != EOF) && (!isspace(c))) Putc(c,fn);
     if (isspace(c)) Ungetc(c,str);
   }
+#if defined(_WIN32) || defined(MACSWIG)
+  /* accept Unix path separator on non-Unix systems */
+  Replaceall(fn, "/", SWIG_FILE_DELIMETER);
+#endif
+#if defined(__CYGWIN__)
+  /* accept Windows path separator in addition to Unix path separator */
+  Replaceall(fn, "\\", SWIG_FILE_DELIMETER);
+#endif
   Seek(fn,0,SEEK_SET);
   return fn;
 }
 
+static String *
+get_options(String *str) {
+
+  int  c;
+  skip_whitespace(str,0);
+  c = Getc(str);
+  if (c == '(') {
+    String *opt;
+    int     level = 1;
+    opt = NewString("(");
+    while (((c = Getc(str)) != EOF)) {
+      Putc(c,opt);
+      if (c == ')') {
+	level--;
+	if (!level) return opt;
+      }
+      if (c == '(') level++;
+    }
+    Delete(opt);
+    return 0;
+  } else {
+    Ungetc(c,str);
+    return 0;
+  }
+}
 /* -----------------------------------------------------------------------------
  * expand_macro()
  *
@@ -459,14 +545,13 @@ get_filename(String *str) {
  * of error occurred.
  * ----------------------------------------------------------------------------- */
 
-DOH *expanded_value = 0;
-
 static String *
 expand_macro(String_or_char *name, List *args)
 {
   DOH *symbols, *ns, *macro, *margs, *mvalue, *temp, *tempa, *e;
   DOH *Preprocessor_replace(DOH *);
   int i, l;
+  int isvarargs = 0;
 
   symbols = Getattr(cpp,"symbols");
   if (!symbols) return 0;
@@ -495,17 +580,45 @@ expand_macro(String_or_char *name, List *args)
   assert(mvalue);
   margs = Getattr(macro,"args");
 
+  if (args && Getattr(macro,"varargs")) {
+    isvarargs = 1;
+    /* Variable length argument macro.  We need to collect all of the extra arguments into a single argument */
+    if (Len(args) >= (Len(margs)-1)) {
+      int i;
+      int vi, na;
+      String *vararg = NewString("");
+      vi = Len(margs)-1;
+      na = Len(args);
+      for (i = vi; i < na; i++) {
+	Append(vararg,Getitem(args,i));
+	if ((i+1) < na) {
+	  Append(vararg,",");
+	}
+      }
+      /* Remove arguments */
+      for (i = vi; i < na; i++) {
+	Delitem(args,vi);
+      }
+      Append(args,vararg);
+      Delete(vararg);
+    }
+  }
   /* If there are arguments, see if they match what we were given */
-  if ((margs) && (Len(margs) != Len(args))) {
-    if (Len(margs) > 1)
-      cpp_error(Getfile(args),Getline(args),"Macro '%s' expects %d arguments\n", name, Len(margs));
-    else if (Len(margs) == 1)
-      cpp_error(Getfile(args),Getline(args),"Macro '%s' expects 1 argument\n", name);
+  if (args && (margs) && (Len(margs) != Len(args))) {
+    if (Len(margs) > (1+isvarargs))
+      Swig_error(Getfile(args),Getline(args),"Macro '%s' expects %d arguments\n", name, Len(margs)-isvarargs);
+    else if (Len(margs) == (1+isvarargs))
+      Swig_error(Getfile(args),Getline(args),"Macro '%s' expects 1 argument\n", name);
     else
-      cpp_error(Getfile(args),Getline(args),"Macro '%s' expects no arguments\n", name);
+      Swig_error(Getfile(args),Getline(args),"Macro '%s' expects no arguments\n", name);
     return 0;
   }
 
+  /* If the macro expects arguments, but none were supplied, we leave it in place */
+  if (!args && (margs)) {
+    return NewString(name);
+  }
+
   /* Copy the macro value */
   ns = Copy(mvalue);
   copy_location(mvalue,ns);
@@ -513,19 +626,15 @@ expand_macro(String_or_char *name, List *args)
   /* Tag the macro as being expanded.   This is to avoid recursion in
      macro expansion */
 
-  if (!expanded_value) {
-    expanded_value = NewString("");
-    DohIntern(expanded_value);
-  }
-  Setattr(macro,"*expanded*",expanded_value);
-
   temp = NewString("");
   tempa = NewString("");
-  if (margs) {
+  if (args && margs) {
     l = Len(margs);
     for (i = 0; i < l; i++) {
       DOH *arg, *aname;
+      String *reparg;
       arg = Getitem(args,i);           /* Get an argument value */
+      reparg = Preprocessor_replace(arg);
       aname = Getitem(margs,i);        /* Get macro argument name */
       if (strstr(Char(ns),"\001")) {
 	/* Try to replace a quoted version of the argument */
@@ -533,7 +642,20 @@ expand_macro(String_or_char *name, List *args)
 	Clear(tempa);
 	Printf(temp,"\001%s", aname);
 	Printf(tempa,"\"%s\"",arg);
-	Replace(ns, temp, tempa, DOH_REPLACE_ANY);
+	Replace(ns, temp, tempa, DOH_REPLACE_ID_END);
+      }
+      if (strstr(Char(ns),"\002")) {
+	/* Look for concatenation tokens */
+	Clear(temp);
+	Clear(tempa);
+	Printf(temp,"\002%s",aname);
+	Append(tempa,"\002\003");
+	Replace(ns, temp, tempa, DOH_REPLACE_ID_END);
+	Clear(temp);
+	Clear(tempa);
+	Printf(temp,"%s\002",aname);
+	Append(tempa,"\003\002");
+	Replace(ns,temp,tempa, DOH_REPLACE_ID_BEGIN);
       }
 
       /* Non-standard macro expansion.   The value `x` is replaced by a quoted
@@ -555,22 +677,72 @@ expand_macro(String_or_char *name, List *args)
 	}
 	Replace(ns,temp,rep, DOH_REPLACE_ANY);
       }
-      Replace(ns, aname, arg, DOH_REPLACE_ID);
+
+      /* Non-standard mangle expansions.  
+	 The #@Name is replaced by mangle_arg(Name). */
+      if (strstr(Char(ns),"\004")) {
+	String* marg = Swig_string_mangle(arg);
+	Clear(temp);
+	Printf(temp,"\004%s", aname);
+	Replace(ns, temp, marg, DOH_REPLACE_ID_END);
+	Delete(marg);
+      }
+      if (strstr(Char(ns),"\005")) {
+	String* marg = Swig_string_mangle(arg);
+	Clear(temp);
+	Clear(tempa);
+	Printf(temp,"\005%s", aname);
+	Printf(tempa,"\"%s\"", marg);
+	Replace(ns, temp, tempa, DOH_REPLACE_ID_END);
+	Delete(marg);
+      }
+
+      if (isvarargs && i == l-1 && Len(arg) == 0) {
+	/* Zero length varargs macro argument.   We search for commas that might appear before and nuke them */
+	char *a, *s, *t, *name;
+        int namelen;
+	s = Char(ns);
+        name = Char(aname);
+        namelen = Len(aname);
+	a = strstr(s,name);
+	while (a) {
+          if (!isidchar(a[namelen+1])) {
+          /* Matched the entire vararg name, not just a prefix */
+	    t = a-1;
+	    if (*t == '\002') {
+	      t--;
+	      while (t >= s) {
+		if (isspace((int) *t)) t--;
+		else if (*t == ',') {
+		  *t = ' ';
+		} else break;
+	      }
+	    }
+	  }
+	  a = strstr(a+namelen,name);
+	}
+      }
+      /*      Replace(ns, aname, arg, DOH_REPLACE_ID); */
+      Replace(ns, aname, reparg, DOH_REPLACE_ID);   /* Replace expanded args */
+      Replace(ns, "\003", arg, DOH_REPLACE_ANY);    /* Replace unexpanded arg */
+      Delete(reparg);
     }
   }
   Replace(ns,"\002","",DOH_REPLACE_ANY);    /* Get rid of concatenation tokens */
   Replace(ns,"\001","#",DOH_REPLACE_ANY);   /* Put # back (non-standard C) */
-
+  Replace(ns,"\004","#@",DOH_REPLACE_ANY);   /* Put # back (non-standard C) */
 
   /* Expand this macro even further */
+  Setattr(macro,"*expanded*","1"); 
+
   e = Preprocessor_replace(ns);
 
-  Delete(ns);
   Delattr(macro,"*expanded*");
+  Delete(ns);
+
   if (Getattr(macro,"swigmacro")) {
     String *g;
     String *f = NewString("");
-    /*    Printf(f," %%macro \"%s\",\"%s\",%d\n ", name, Getfile(macro), Getline(macro)); */
     Seek(e,0,SEEK_SET);
     copy_location(macro,e);
     g = Preprocessor_parse(e);
@@ -588,6 +760,23 @@ expand_macro(String_or_char *name, List *args)
   return e;
 }
 
+/* -----------------------------------------------------------------------------
+ * evaluate_args()
+ *
+ * Evaluate the arguments of a macro 
+ * ----------------------------------------------------------------------------- */
+
+List *evaluate_args(List *x) {
+  Iterator i;
+  String *Preprocessor_replace(String *);
+  List *nl = NewList();
+
+  for (i = First(x); i.item; i = Next(i)) {
+    Append(nl,Preprocessor_replace(i.item));
+  }
+  return nl;
+}
+
 /* -----------------------------------------------------------------------------
  * DOH *Preprocessor_replace(DOH *s)
  *
@@ -653,6 +842,7 @@ Preprocessor_replace(DOH *s)
 	    DOH *arg = 0;
 	    args = NewList();
 	    arg = NewString("");
+	    if (isidchar(c)) Putc(c,arg);
 	    while ((c = Getc(s)) != EOF) {
 	      if (!isidchar(c)) {
 		Seek(s,-1,SEEK_CUR);
@@ -663,8 +853,8 @@ Preprocessor_replace(DOH *s)
 	    Append(args,arg);
 	    Delete(arg);
 	  }
-	  if (!args) {
-	    cpp_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
+	  if ((!args) || (!Len(args))) {
+	    Swig_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
 	    state = 0;
 	    break;
 	  }
@@ -686,7 +876,10 @@ Preprocessor_replace(DOH *s)
 	  break;
 	}
 	if (Cmp(id,"__FILE__") == 0) {
-	  Printf(ns,"\"%s\"",Getfile(s));
+	  String *fn = Copy(Getfile(s));
+	  Replaceall(fn,"\\","\\\\");
+	  Printf(ns,"\"%s\"",fn);
+	  Delete(fn);
 	  state = 0;
 	  break;
 	}
@@ -698,6 +891,10 @@ Preprocessor_replace(DOH *s)
 	  if (Getattr(m,"args")) {
 	    /* Yep.  We need to go find the arguments and do a substitution */
 	    args = find_args(s);
+	    if (!Len(args)) {
+	      Delete(args);
+	      args = 0;
+	    }
 	  } else {
 	    args = 0;
 	  }
@@ -746,14 +943,14 @@ Preprocessor_replace(DOH *s)
   if (state == 1) {
     /* See if this is the special "defined" macro */
     if (Cmp(id,"defined") == 0) {
-      cpp_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
+      Swig_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
     } else if ((m = Getattr(symbols,id))) {
 	DOH *e;
 	/* Yes.  There is a macro here */
 	/* See if the macro expects arguments */
-	if (Getattr(m,"args")) {
-	  cpp_error(Getfile(id),Getline(id),"Macro arguments expected.\n");
-	}
+	/*	if (Getattr(m,"args")) {
+	  Swig_error(Getfile(id),Getline(id),"Macro arguments expected.\n");
+	  } */
 	e = expand_macro(id,0);
 	Printf(ns,"%s",e);
 	Delete(e);
@@ -777,9 +974,9 @@ static int
 check_id(DOH *s)
 {
   static SwigScanner *scan = 0;
+  int c;
+  int hastok = 0;
 
-  int c, state = 0;
-  int hasvalue = 0;
   Seek(s,0,SEEK_SET);
 
   if (!scan) {
@@ -787,12 +984,15 @@ check_id(DOH *s)
   }
 
   SwigScanner_clear(scan);
-  SwigScanner_push(scan,Copy(s));
+  s = Copy(s);
+  Seek(s,SEEK_SET,0);
+  SwigScanner_push(scan,s);
   while ((c = SwigScanner_token(scan))) {
-    if (c == SWIG_TOKEN_ID) return 1;
+    hastok = 1;
+    if ((c == SWIG_TOKEN_ID) || (c == SWIG_TOKEN_LBRACE) || (c == SWIG_TOKEN_RBRACE)) return 1;
   }
+  if (!hastok) return 1;
   return 0;
-
 }
 
 /* addline().  Utility function for adding lines to a chunk */
@@ -895,12 +1095,12 @@ Preprocessor_parse(String *s)
       else if (c == '\"') {
 	start_line = Getline(s);
 	if (skip_tochar(s,'\"',chunk) < 0) {
-	  cpp_error(Getfile(s),-1,"Unterminated string constant starting at line %d\n",start_line);
+	  Swig_error(Getfile(s),-1,"Unterminated string constant starting at line %d\n",start_line);
 	}
       } else if (c == '\'') {
 	start_line = Getline(s);
 	if (skip_tochar(s,'\'',chunk) < 0) {
-	  cpp_error(Getfile(s),-1,"Unterminated character constant starting at line %d\n",start_line);
+	  Swig_error(Getfile(s),-1,"Unterminated character constant starting at line %d\n",start_line);
 	}
       }
       else if (c == '/') state = 30;  /* Comment */
@@ -949,8 +1149,13 @@ Preprocessor_parse(String *s)
 	if (c == '\n') {
 	  Ungetc(c,s);
 	  state = 50;
+	} else {
+	  state = 42;
+	  if (!isspace(c)) {
+	    Ungetc(c,s);
+	  }
 	}
-	else state = 42;
+
 	copy_location(s,value);
 	break;
       }
@@ -988,8 +1193,12 @@ Preprocessor_parse(String *s)
       break;
 
     case 44:
-      if (c == '\n') cpp_lines++;
-      Putc(c,value);
+      if (c == '\n') {
+	Putc(c,value);
+	cpp_lines++;
+      } else {
+	Ungetc(c,s);
+      }
       state = 43;
       break;
 
@@ -1041,9 +1250,9 @@ Preprocessor_parse(String *s)
 	  if ((m) && !(Getattr(m,"args"))) {
 	    v = Copy(Getattr(m,"value"));
 	    if (Len(v)) {
-	      silent_errors = 1;
+	      Swig_error_silent(1);
 	      v1 = Preprocessor_replace(v);
-	      silent_errors = 0;
+	      Swig_error_silent(0);
 	      /*	      Printf(stdout,"checking '%s'\n", v1); */
 	      if (!check_id(v1)) {
 		if (Len(comment) == 0)
@@ -1080,20 +1289,20 @@ Preprocessor_parse(String *s)
 	}
       } else if (Cmp(id,"else") == 0) {
 	if (level <= 0) {
-	  cpp_error(Getfile(s),Getline(id),"Misplaced #else.\n");
+	  Swig_error(Getfile(s),Getline(id),"Misplaced #else.\n");
 	} else {
 	  cond_lines[level-1] = Getline(id);
 	  if (allow) {
 	    allow = 0;
 	    mask = 0;
 	  } else if (level == start_level) {
-	    allow = 1;
+	    allow = 1*mask;
 	  }
 	}
       } else if (Cmp(id,"endif") == 0) {
 	level--;
 	if (level < 0) {
-	  cpp_error(Getfile(id),Getline(id),"Extraneous #endif ignored.\n");
+	  Swig_error(Getfile(id),Getline(id),"Extraneous #endif.\n");
 	  level = 0;
 	} else {
 	  if (level < start_level) {
@@ -1108,11 +1317,11 @@ Preprocessor_parse(String *s)
 	  start_level = level;
 	  sval = Preprocessor_replace(value);
 	  Seek(sval,0,SEEK_SET);
-	  /*	  Printf(stdout,"Evaluating '%s'\n", sval);*/
+	  /*	  Printf(stdout,"Evaluating '%s'\n", sval); */
   	  val = Preprocessor_expr(sval,&e);
   	  if (e) {
   	    Seek(value,0,SEEK_SET);
-	    /*	    cpp_error(Getfile(value),Getline(value),"Could not evaluate '%s'\n", value); */
+	    Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);
   	    allow = 0;
   	  } else {
   	    if (val == 0)
@@ -1122,7 +1331,7 @@ Preprocessor_parse(String *s)
   	}
       } else if (Cmp(id,"elif") == 0) {
   	if (level == 0) {
-  	  cpp_error(Getfile(s),Getline(id),"Misplaced #elif.\n");
+  	  Swig_error(Getfile(s),Getline(id),"Misplaced #elif.\n");
   	} else {
   	  cond_lines[level-1] = Getline(id);
   	  if (allow) {
@@ -1134,7 +1343,7 @@ Preprocessor_parse(String *s)
   	    val = Preprocessor_expr(sval,&e);
   	    if (e) {
   	      Seek(value,0,SEEK_SET);
-  	      /*      cpp_error(Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);  */
+  	      Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);
   	      allow = 0;
   	    } else {
   	      if (val)
@@ -1147,26 +1356,47 @@ Preprocessor_parse(String *s)
       } else if (Cmp(id,"line") == 0) {
       } else if (Cmp(id,"include") == 0) {
   	if (((include_all) || (import_all)) && (allow)) {
-  	  DOH *s1, *s2, *fn;
+  	  String *s1, *s2, *fn;
+	  char *dirname;
   	  Seek(value,0,SEEK_SET);
   	  fn = get_filename(value);
 	  s1 = cpp_include(fn);
 	  if (s1) {
 	    if (include_all) 
-	      Printf(ns,"%%includefile \"%s\" {\n", Swig_last_file());
+	      Printf(ns,"%%includefile \"%s\" [\n", Swig_last_file());
 	    else if (import_all) 
-	      Printf(ns,"%%importfile \"%s\" {\n", Swig_last_file());
+	      Printf(ns,"%%importfile \"%s\" [\n", Swig_last_file());
+
+	    /* See if the filename has a directory component */
+	    dirname = Swig_file_dirname(Swig_last_file());
+	    if (!strlen(dirname)) dirname = 0;
+	    if (dirname) {
+	      dirname[strlen(dirname)-1] = 0;   /* Kill trailing directory delimeter */
+	      Swig_push_directory(dirname);
+	    }
   	    s2 = Preprocessor_parse(s1);
   	    addline(ns,s2,allow);
-  	    Printf(ns,"\n}\n");
+  	    Printf(ns,"\n]");
+	    if (dirname) {
+	      Swig_pop_directory();
+	    }
 	    Delete(s2);
   	  }
 	  Delete(s1);
  	  Delete(fn);
   	}
       } else if (Cmp(id,"pragma") == 0) {
+	if (Strncmp(value,"SWIG ",5) == 0) {
+	  char *c = Char(value)+5;
+	  while (*c && (isspace((int)*c))) c++;
+	  if (*c) {
+	    if (Strncmp(c,"nowarn=",7) == 0) {
+	      Swig_warnfilter(c+7,1);
+	    }
+	  }
+	}
       } else if (Cmp(id,"level") == 0) {
-	cpp_error(Getfile(s),Getline(id),"cpp debug: level = %d, startlevel = %d\n", level, start_level);
+	Swig_error(Getfile(s),Getline(id),"cpp debug: level = %d, startlevel = %d\n", level, start_level);
       }
       for (i = 0; i < cpp_lines; i++)
   	Putc('\n',ns);
@@ -1242,30 +1472,41 @@ Preprocessor_parse(String *s)
   	if ((Cmp(decl,"%include") == 0) || (Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
   	  /* Got some kind of file inclusion directive  */
   	  if (allow) {
-  	    DOH *s1, *s2, *fn;
+  	    DOH *s1, *s2, *fn, *opt;
 
 	    if (Cmp(decl,"%extern") == 0) {
-	      Printf(stderr,"%s:%d. %%extern is deprecated. Use %%import instead.\n",Getfile(s),Getline(s));
+	      Swig_warning(WARN_DEPRECATED_EXTERN, Getfile(s),Getline(s),"%%extern is deprecated. Use %%import instead.\n");
 	      Clear(decl);
 	      Printf(decl,"%%import");
 	    }
+	    opt = get_options(s);
   	    fn = get_filename(s);
 	    s1 = cpp_include(fn);
 	    if (s1) {
+	      char *dirname;
   	      add_chunk(ns,chunk,allow);
   	      copy_location(s,chunk);
-  	      Printf(ns,"%sfile \"%s\" {\n", decl, Swig_last_file());
+  	      Printf(ns,"%sfile%s \"%s\" [\n", decl, opt, Swig_last_file());
 	      if ((Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
 		Preprocessor_define("WRAPEXTERN 1", 0);
 		Preprocessor_define("SWIGIMPORT 1", 0);
 	      }
+	      dirname = Swig_file_dirname(Swig_last_file());
+	      if (!strlen(dirname)) dirname = 0;
+	      if (dirname) {
+		dirname[strlen(dirname)-1] = 0;   /* Kill trailing directory delimeter */
+		Swig_push_directory(dirname);
+	      }
   	      s2 = Preprocessor_parse(s1);
+	      if (dirname) {
+		Swig_pop_directory();
+	      }
 	      if ((Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
 		Preprocessor_undef("SWIGIMPORT");
 		Preprocessor_undef("WRAPEXTERN");
 	      }
   	      addline(ns,s2,allow);
-  	      Printf(ns,"\n}\n");
+  	      Printf(ns,"\n]");
 	      Delete(s2);
 	      Delete(s1);
   	    }
@@ -1326,18 +1567,18 @@ Preprocessor_parse(String *s)
     }
   }
   while (level > 0) {
-    cpp_error(Getfile(s),-1,"Missing #endif for conditional starting on line %d\n", cond_lines[level-1]);
+    Swig_error(Getfile(s),-1,"Missing #endif for conditional starting on line %d\n", cond_lines[level-1]);
     level--;
   }
   if (state == 150) {
     Seek(value,0,SEEK_SET);
-    cpp_error(Getfile(s),-1,"Missing %%enddef for macro starting on line %d\n",Getline(value));
+    Swig_error(Getfile(s),-1,"Missing %%enddef for macro starting on line %d\n",Getline(value));
   }
   if ((state >= 105) && (state < 107)) {
-    cpp_error(Getfile(s),-1,"Unterminated %%{ ... %%} block starting on line %d\n", start_line);
+    Swig_error(Getfile(s),-1,"Unterminated %%{ ... %%} block starting on line %d\n", start_line);
   }
   if ((state >= 30) && (state < 40)) {
-    cpp_error(Getfile(s),-1,"Unterminated comment starting on line %d\n", start_line);
+    Swig_error(Getfile(s),-1,"Unterminated comment starting on line %d\n", start_line);
   }
   add_chunk(ns,chunk,allow);
   copy_location(s,chunk);
