@@ -10,8 +10,9 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Preprocessor/expr.c,v 1.6.4.1 2001/12/21 02:56:26 beazley Exp $";
+char cvsroot_expr_c[] = "$Header: /cvsroot/swig/SWIG/Source/Preprocessor/expr.c,v 1.10 2004/01/15 00:58:13 beazley Exp $";
 
+#include "swig.h"
 #include "preprocessor.h"
 
 static SwigScanner *scan = 0;
@@ -19,6 +20,7 @@ static SwigScanner *scan = 0;
 typedef struct {
   int       op;
   long      value;
+  String   *svalue;
 } exprval;
 
 #define  EXPR_TOP      1
@@ -66,95 +68,122 @@ static void reduce_op() {
     sp = 0;
     return;
   }
-  switch(stack[sp-1].value) {
-  case SWIG_TOKEN_STAR:
-    stack[sp-2].value = stack[sp-2].value * stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_EQUALTO:
-    stack[sp-2].value = stack[sp-2].value == stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_NOTEQUAL:
-    stack[sp-2].value = stack[sp-2].value != stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_PLUS:
-    stack[sp-2].value = stack[sp-2].value + stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_MINUS:
-    stack[sp-2].value = stack[sp-2].value - stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_AND:
-    stack[sp-2].value = stack[sp-2].value & stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_LAND:
-    stack[sp-2].value = stack[sp-2].value && stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_OR:
-    stack[sp-2].value = stack[sp-2].value | stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_LOR:
-    stack[sp-2].value = stack[sp-2].value || stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_XOR:
-    stack[sp-2].value = stack[sp-2].value ^ stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_LESSTHAN:
-    stack[sp-2].value = stack[sp-2].value < stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_GREATERTHAN:
-    stack[sp-2].value = stack[sp-2].value > stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_LTEQUAL:
-    stack[sp-2].value = stack[sp-2].value <= stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_GTEQUAL:
-    stack[sp-2].value = stack[sp-2].value >= stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_NOT:
-    stack[sp-1].value = ~stack[sp].value;
-    sp--;
-    break;
-  case SWIG_TOKEN_LNOT:
-    stack[sp-1].value = !stack[sp].value;
-    sp--;
-    break;
-  case EXPR_UMINUS:
-    stack[sp-1].value = -stack[sp].value;
-    sp--;
-    break;
-  case SWIG_TOKEN_SLASH:
-    stack[sp-2].value = stack[sp-2].value / stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_PERCENT:
-    stack[sp-2].value = stack[sp-2].value % stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_LSHIFT:
-    stack[sp-2].value = stack[sp-2].value << stack[sp].value;
-    sp -= 2;
-    break;
-  case SWIG_TOKEN_RSHIFT:
-    stack[sp-2].value = stack[sp-2].value >> stack[sp].value;
-    sp -= 2;
-    break;
-  default:
-    errmsg = "Syntax error";
-    sp = 0;
-    break;
+  if (stack[sp-2].svalue || stack[sp].svalue) {
+    /* A string expression */
+    if (!(stack[sp-2].svalue && stack[sp].svalue)) {
+      errmsg = "Can't mix strings and integers in expression";
+      sp = 0;
+      return;
+    }
+    switch(stack[sp-1].value) {
+    case SWIG_TOKEN_EQUALTO:
+      stack[sp-2].value = (Strcmp(stack[sp-2].svalue,stack[sp].svalue) == 0);
+      Delete(stack[sp-2].svalue);
+      Delete(stack[sp].svalue);
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_NOTEQUAL:
+      stack[sp-2].value = (Strcmp(stack[sp-2].svalue,stack[sp].svalue) != 0);
+      Delete(stack[sp-2].svalue);
+      Delete(stack[sp].svalue);
+      sp -= 2;
+      break;
+    default:
+      errmsg = "Syntax error";
+      sp = 0;
+      break;
+    }
+  } else {
+    switch(stack[sp-1].value) {
+    case SWIG_TOKEN_STAR:
+      stack[sp-2].value = stack[sp-2].value * stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_EQUALTO:
+      stack[sp-2].value = stack[sp-2].value == stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_NOTEQUAL:
+      stack[sp-2].value = stack[sp-2].value != stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_PLUS:
+      stack[sp-2].value = stack[sp-2].value + stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_MINUS:
+      stack[sp-2].value = stack[sp-2].value - stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_AND:
+      stack[sp-2].value = stack[sp-2].value & stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_LAND:
+      stack[sp-2].value = stack[sp-2].value && stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_OR:
+      stack[sp-2].value = stack[sp-2].value | stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_LOR:
+      stack[sp-2].value = stack[sp-2].value || stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_XOR:
+      stack[sp-2].value = stack[sp-2].value ^ stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_LESSTHAN:
+      stack[sp-2].value = stack[sp-2].value < stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_GREATERTHAN:
+      stack[sp-2].value = stack[sp-2].value > stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_LTEQUAL:
+      stack[sp-2].value = stack[sp-2].value <= stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_GTEQUAL:
+      stack[sp-2].value = stack[sp-2].value >= stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_NOT:
+      stack[sp-1].value = ~stack[sp].value;
+      sp--;
+      break;
+    case SWIG_TOKEN_LNOT:
+      stack[sp-1].value = !stack[sp].value;
+      sp--;
+      break;
+    case EXPR_UMINUS:
+      stack[sp-1].value = -stack[sp].value;
+      sp--;
+      break;
+    case SWIG_TOKEN_SLASH:
+      stack[sp-2].value = stack[sp-2].value / stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_PERCENT:
+      stack[sp-2].value = stack[sp-2].value % stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_LSHIFT:
+      stack[sp-2].value = stack[sp-2].value << stack[sp].value;
+      sp -= 2;
+      break;
+    case SWIG_TOKEN_RSHIFT:
+      stack[sp-2].value = stack[sp-2].value >> stack[sp].value;
+      sp -= 2;
+      break;
+    default:
+      errmsg = "Syntax error";
+      sp = 0;
+      break;
+    }
   }
   stack[sp].op = EXPR_VALUE;
 }
@@ -171,6 +200,19 @@ Preprocessor_expr_init (void) {
   if (!scan) scan = NewSwigScanner();
 }
 
+/* -----------------------------------------------------------------------------
+ * Tokenizer 
+ * ----------------------------------------------------------------------------- */
+
+static int expr_token(SwigScanner *s) {
+  int t;
+  while(1) {
+    t = SwigScanner_token(s);
+    if (!((t == SWIG_TOKEN_BACKSLASH) || (t == SWIG_TOKEN_ENDLINE))) break;
+  }
+  return t;
+}
+
 /* -----------------------------------------------------------------------------
  * Preprocessor_expr()
  *
@@ -187,6 +229,7 @@ Preprocessor_expr(DOH *s, int *error) {
   assert(scan);
 
   Seek(s,0,SEEK_SET);
+  /* Printf(stdout,"evaluating : '%s'\n", s); */
   *error = 0;
   SwigScanner_clear(scan);
   SwigScanner_push(scan,s);
@@ -200,7 +243,7 @@ Preprocessor_expr(DOH *s, int *error) {
     switch(stack[sp].op) {
     case EXPR_TOP:
       /* An expression.   Can be a number or another expression enclosed in parens */
-      token = SwigScanner_token(scan);
+      token = expr_token(scan);
       if (!token) {
 	errmsg = "Expected an expression";
 	*error = 1;
@@ -210,6 +253,7 @@ Preprocessor_expr(DOH *s, int *error) {
 	/* A number.  Reduce EXPR_TOP to an EXPR_VALUE */
 	char *c = Char(SwigScanner_text(scan));
 	stack[sp].value = (long) strtol(c,0,0);
+	stack[sp].svalue = 0;
 	/*	  stack[sp].value = (long) atol(Char(SwigScanner_text(scan))); */
 	stack[sp].op = EXPR_VALUE;
       } else if (token == SWIG_TOKEN_PLUS) { }
@@ -218,16 +262,25 @@ Preprocessor_expr(DOH *s, int *error) {
 	stack[sp].value = token;
 	stack[sp++].op = EXPR_OP;
 	stack[sp].op = EXPR_TOP;
+	stack[sp].svalue = 0;
       } else if ((token == SWIG_TOKEN_LPAREN)) {
 	stack[sp++].op = EXPR_GROUP;
 	stack[sp].op = EXPR_TOP;
 	stack[sp].value = 0;
-      } else if (token == SWIG_TOKEN_ENDLINE) { }
-      else goto syntax_error;
+	stack[sp].svalue = 0;
+      } else if (token == SWIG_TOKEN_ENDLINE) { 
+      } else if ((token == SWIG_TOKEN_STRING)) {
+	stack[sp].svalue = NewString(SwigScanner_text(scan));
+	stack[sp].op = EXPR_VALUE;
+      } else if ((token == SWIG_TOKEN_ID)) {
+	stack[sp].value = 0;
+	stack[sp].svalue = 0;
+	stack[sp].op = EXPR_VALUE;
+      } else goto syntax_error;
       break;
     case EXPR_VALUE:
       /* A value is on the stack.   We may reduce or evaluate depending on what the next token is */
-      token = SwigScanner_token(scan);
+      token = expr_token(scan);
       if (!token) {
 	/* End of input. Might have to reduce if an operator is on stack */
 	while (sp > 0) {
@@ -304,6 +357,7 @@ Preprocessor_expr(DOH *s, int *error) {
 	break;
       }
       break;
+
     default:
       fprintf(stderr,"Internal error in expression evaluator.\n");
       abort();
