@@ -13,7 +13,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cwrap_c[] = "$Header: /cvsroot/SWIG/Source/Swig/cwrap.c,v 1.30 2003/09/23 21:26:31 cheetah Exp $";
+char cvsroot_cwrap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/cwrap.c,v 1.39 2004/06/03 22:57:27 wsfulton Exp $";
 
 #include "swig.h"
 
@@ -85,18 +85,30 @@ Swig_clocal(SwigType *t, String_or_char *name, String_or_char *value) {
  * This function only converts user defined types to pointers.
  * ----------------------------------------------------------------------------- */
 
+static int varref = 0;
 String *
 Swig_wrapped_var_type(SwigType *t) {
   SwigType *ty;
-  ty = Copy(t);
+
+  if (!Strstr(t,"enum $unnamed")) {
+    ty = Copy(t);
+  } else {
+    /* Change the type for unnamed enum instance variables */
+    ty = NewString("int");
+  }
 
   if (SwigType_isclass(t)) {
-    SwigType_add_pointer(ty);
+    if (varref) {
+      if (!SwigType_isconst(ty)) SwigType_add_qualifier(ty, "const");
+      SwigType_add_reference(ty);
+    } else {
+      SwigType_add_pointer(ty);
+    }
   }
   return ty;
 }
 
-String *
+static String *
 Swig_wrapped_var_deref(SwigType *t, String_or_char *name) {
   if (SwigType_isclass(t)) {
     return NewStringf("*%s",name);
@@ -105,10 +117,15 @@ Swig_wrapped_var_deref(SwigType *t, String_or_char *name) {
   }
 }
 
-String *
-Swig_wrapped_var_assign(SwigType *t, String_or_char *name) {
+static String *
+Swig_wrapped_var_assign(SwigType *t, const String_or_char *name) {
   if (SwigType_isclass(t)) {
-    return NewStringf("&%s",name);
+    if (varref) {
+      String* ty = SwigType_lstr(t,0);
+      return NewStringf("(const %s&)%s",ty, name);
+    } else {
+      return NewStringf("&%s",name);
+    }
   } else {
     return SwigType_lcaststr(t,name);
   }
@@ -289,11 +306,8 @@ Swig_cmethod_call(String_or_char *name, ParmList *parms, String_or_char *self) {
     Replaceall(func,"this", SwigType_rcaststr(pt, Swig_cparm_name(p,0)));
   }
 
-  if (SwigType_istemplate(name)) {
-      Printf(func,"template %s(", nname);
-  } else {
-      Printf(func,"%s(", nname);
-  }
+  Printf(func,"%s(", nname);
+
   i++;
   p = nextSibling(p);
   while (p) {
@@ -395,6 +409,106 @@ Swig_cppconstructor_director_call(String_or_char *name, ParmList *parms) {
   return Swig_cppconstructor_base_call(name, parms, 0);
 }
 
+/* -----------------------------------------------------------------------------
+ * Swig_cattr_search()
+ *
+ * This function searches for the class attribute 'attr' in the class
+ * 'n' or recursively in its bases.
+ *
+ * If you define SWIG_FAST_REC_SEARCH, the method will set the found
+ * 'attr' in io the target class 'n'. If not, the method will set the
+ * 'noattr' one. This prevents of having to navigate the entire
+ * hierarchy tree everytime, so, it is an O(1) method...  or something
+ * like that. However, it populates all the parsed classes with the
+ * 'attr' and/or 'noattr' attributes.
+ *
+ * If you undefine the SWIG_FAST_REC_SEARCH no attribute will be set
+ * while searching. This could be slower for large projects with very
+ * large hierarchy trees... or maybe not. But it will be cleaner. 
+ *
+ * Maybe later a swig option can be added to switch at runtime.
+ *
+ * ----------------------------------------------------------------------------- */
+
+/* #define SWIG_FAST_REC_SEARCH 1 */
+String *
+Swig_cattr_search(Node *n, const String *attr, const String *noattr)
+{
+  String *f = 0;
+  n = Swig_methodclass(n);  
+  if (Getattr(n, noattr)) {
+    /* Printf(stdout,"noattr in %s\n", Getattr(n,"name")); */
+    return 0;
+  }
+  f = Getattr(n, attr);
+  if (f) {
+    /* Printf(stdout,"attr in %s\n", Getattr(n,"name")); */
+    return f;
+  } else {
+    List* bl = Getattr(n, "bases");
+    if (bl) {
+      Iterator bi;
+      for (bi = First(bl); bi.item; bi = Next(bi)) {
+	f = Swig_cattr_search(bi.item, attr, noattr);
+	if (f) {
+#ifdef SWIG_FAST_REC_SEARCH
+	  Setattr(n, attr, f);
+#endif
+	  return f;
+	}
+      }
+    }
+  }
+#ifdef SWIG_FAST_REC_SEARCH
+  Setattr(n, noattr, "1");
+#endif
+  return 0;
+}
+
+/* -----------------------------------------------------------------------------
+ * Swig_unref_call()
+ *
+ * find the unref call, if any.
+ * ----------------------------------------------------------------------------- */
+
+String *
+Swig_unref_call(Node *n) {
+  String* unref = 0;
+  n = Swig_methodclass(n);  
+  if (!Getattr(n,"feature:nounref")) {
+    unref = Getattr(n,"feature:unref");
+    unref = unref ? unref : 
+      Swig_cattr_search(n,"feature:unref","feature:nounref");
+    if (unref) {
+      unref = NewStringf("%s",unref);
+      Replaceall(unref,"$this",Swig_cparm_name(0,0));
+    }
+  }
+  return unref;
+}
+
+/* -----------------------------------------------------------------------------
+ * Swig_ref_call()
+ *
+ * find the ref call, if any.
+ * ----------------------------------------------------------------------------- */
+
+String *
+Swig_ref_call(Node *n, const String* lname) {
+  String* ref = 0;
+  n = Swig_methodclass(n);
+  if (!Getattr(n,"feature:noref")) {
+    ref = Getattr(n,"feature:ref");
+    ref = ref ? ref : 
+      Swig_cattr_search(n,"feature:ref","feature:noref");
+    if (ref) {
+      ref = NewStringf("%s",ref);
+      Replaceall(ref,"$this",lname);
+    }
+  }
+  return ref;
+}
+
 /* -----------------------------------------------------------------------------
  * Swig_cdestructor_call()
  *
@@ -404,11 +518,11 @@ Swig_cppconstructor_director_call(String_or_char *name, ParmList *parms) {
  * ----------------------------------------------------------------------------- */
 
 String *
-Swig_cdestructor_call() {
-  String *func;
-  func = NewString("");
-  Printf(func,"free((char *) %s)", Swig_cparm_name(0,0));
-  return func;
+Swig_cdestructor_call(Node *n) {
+  String* unref = Swig_unref_call(n);
+  if (unref) return unref;
+  
+  return NewStringf("free((char *) %s);",Swig_cparm_name(0,0));
 }
 
 
@@ -421,12 +535,11 @@ Swig_cdestructor_call() {
  * ----------------------------------------------------------------------------- */
 
 String *
-Swig_cppdestructor_call() {
-  String *func;
+Swig_cppdestructor_call(Node *n) {
+  String* unref = Swig_unref_call(n);
+  if (unref) return unref;
 
-  func = NewString("");
-  Printf(func,"delete %s", Swig_cparm_name(0,0));
-  return func;
+  return NewStringf("delete %s;",Swig_cparm_name(0,0));
 }
 
 /* -----------------------------------------------------------------------------
@@ -446,7 +559,13 @@ Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self)
   else self = NewString(self);
   Replaceall(self,"this",Swig_cparm_name(0,0));
   if (SwigType_type(type) != T_ARRAY) {
-    Printf(func,"if (%s) %s%s = %s",Swig_cparm_name(0,0), self,name, Swig_wrapped_var_deref(type, Swig_cparm_name(0,1)));
+    if (!Strstr(type,"enum $unnamed")) {
+      Printf(func,"if (%s) %s%s = %s",Swig_cparm_name(0,0), self,name,
+	     Swig_wrapped_var_deref(type, Swig_cparm_name(0,1)));
+    } else {
+      Printf(func,"if (%s && sizeof(int) == sizeof(%s%s)) *(int*)(void*)&(%s%s) = %s",
+             Swig_cparm_name(0,0), self, name, self, name, Swig_cparm_name(0,1));
+    }
   }
   Delete(self);
   return(func);
@@ -463,7 +582,8 @@ Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self)
  * ----------------------------------------------------------------------------- */
 
 String *
-Swig_cmemberget_call(String_or_char *name, SwigType *t, String_or_char *self) {
+Swig_cmemberget_call(const String_or_char *name, SwigType *t,
+		     String_or_char *self) {
   String *func;
   if (!self) self = NewString("(this)->");
   else self = NewString(self);
@@ -507,6 +627,32 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
   SwigType_add_pointer(type);
   p = NewParm(type,"self");
   Setattr(p,"hidden","1");
+  /*
+    Disable the 'this' ownership in 'self' to manage inplace
+    operations like:
+
+    A& A::operator+=(int i) { ...; return *this;}
+     
+    Here the 'self' parameter ownership needs to be disabled since
+    there could be two objects sharing the same 'this' pointer: the
+    input and the result one. And worse, the pointer could be deleted
+    in one of the objects (input), leaving the other (output) with
+    just a seg. fault to happen.
+
+    To avoid the previous problem, use
+
+      %feature("self:disown") *::operator+=;
+      %feature("new") *::operator+=;
+
+    These two lines just transfer the ownership of the 'this' pointer
+    from the input to the output wrapping object.
+    
+    This happens in python, but may also happens in other target
+    languages.
+  */
+  if (Getattr(n,"feature:self:disown")) {
+    Setattr(p,"wrap:disown",Getattr(n,"feature:self:disown"));
+  }
   set_nextSibling(p,parms);
   Delete(type);
   
@@ -584,9 +730,11 @@ Swig_directormethod(Node *n) {
     if (vtable) {
       String *name = Getattr(n, "name");
       String *decl = Getattr(n, "decl");
-      String *method_id = NewStringf("%s|%s", name, decl);
+      String *local_decl = SwigType_typedef_resolve_all(decl);
+      String *method_id = NewStringf("%s|%s", name, local_decl);
       Hash *item = Getattr(vtable, method_id);
       Delete(method_id);
+      Delete(local_decl);
       if (item) {
         return (Getattr(item, "director") != 0);
       }
@@ -789,11 +937,9 @@ Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
     Delete(mangled);
   } else {
     if (cplus) {
-      String* action = NewString("");
-      Printf(action, "%s;\n", Swig_cppdestructor_call());
-      Setattr(n,"wrap:action", action);
+      Setattr(n,"wrap:action", NewStringf("%s\n",Swig_cppdestructor_call(n)));
     } else {
-      Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cdestructor_call()));
+      Setattr(n,"wrap:action", NewStringf("%s\n", Swig_cdestructor_call(n)));
     }
   }
   Setattr(n,"type",type);
@@ -820,6 +966,7 @@ Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   String   *membername;
   String   *mangled;
   String   *self= 0;
+  varref = flags & CWRAP_VAR_REFERENCE;  
 
   if (flags & CWRAP_SMART_POINTER) {
     self = NewString("(*this)->");
@@ -865,6 +1012,7 @@ Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   Delete(membername);
   Delete(mangled);
   Delete(self);
+  varref = 0;
   return SWIG_OK;
 }
 
@@ -884,6 +1032,7 @@ Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   String   *membername;
   String   *mangled;
   String   *self = 0;
+  varref = flags & CWRAP_VAR_REFERENCE;  
 
   if (flags & CWRAP_SMART_POINTER) {
     self = NewString("(*this)->");
@@ -921,6 +1070,7 @@ Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   Delete(ty);
   Delete(membername);
   Delete(mangled);
+  varref = 0;
   return SWIG_OK;
 }
 
@@ -945,7 +1095,11 @@ Swig_VarsetToFunction(Node *n) {
   parms = NewParm(ty,"value");
   Delete(ty);
   
-  Setattr(n,"wrap:action", NewStringf("%s = %s;\n", nname, Swig_wrapped_var_deref(type,Swig_cparm_name(0,0))));
+  if (!Strstr(type,"enum $unnamed")) {
+    Setattr(n,"wrap:action", NewStringf("%s = %s;\n", nname, Swig_wrapped_var_deref(type,Swig_cparm_name(0,0))));
+  } else {
+    Setattr(n,"wrap:action", NewStringf("if (sizeof(int) == sizeof(%s)) *(int*)(void*)&(%s) = %s;\n", nname, nname, Swig_cparm_name(0,0)));
+  }
   Setattr(n,"type","void");
   Setattr(n,"parms",parms);
   Delete(parms);
