@@ -11,7 +11,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/include.c,v 1.10.4.4 2002/01/07 04:08:51 beazley Exp $";
+char cvsroot_include_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/include.c,v 1.19 2003/12/28 21:38:58 cheetah Exp $";
 
 #include "swig.h"
 
@@ -68,6 +68,37 @@ Swig_add_directory(const String_or_char *dirname) {
   Append(directories, dirname);
 }
 
+/* -----------------------------------------------------------------------------
+ * Swig_push_directory()
+ *
+ * Inserts a directory at the front of the SWIG search path.  This is used by
+ * the preprocessor to grab files in the same directory as other included files.
+ * ----------------------------------------------------------------------------- */
+
+void 
+Swig_push_directory(const String_or_char *dirname) {
+  if (!directories) directories = NewList();
+  assert(directories);
+  if (!DohIsString(dirname)) {
+    dirname = NewString((char *) dirname);
+    assert(dirname);
+  }
+  Insert(directories, 0, dirname);
+}
+
+/* -----------------------------------------------------------------------------
+ * Swig_pop_directory()
+ *
+ * Pops a directory off the front of the SWIG search path.  This is used by
+ * the preprocessor.
+ * ----------------------------------------------------------------------------- */
+
+void 
+Swig_pop_directory() {
+  if (!directories) return;
+  Delitem(directories,0);
+}
+
 /* -----------------------------------------------------------------------------
  * Swig_last_file()
  * 
@@ -145,6 +176,10 @@ Swig_open(const String_or_char *name) {
       Delete(spath);
   }
   if (f) {
+#if defined(_WIN32) /* Note not on Cygwin else filename is displayed with double '/' */
+    Replaceall(filename,"\\\\","\\"); /* remove double '\' in case any already present */
+    Replaceall(filename,"\\","\\\\");
+#endif
     Delete(lastpath);
     lastpath = Copy(filename);
   }
@@ -251,12 +286,12 @@ Swig_file_suffix(const String_or_char *filename) {
   char *d;
   char *c = Char(filename);
   if (strlen(c)) {
-    d = c + strlen(filename) - 1;
+    d = c + Len(filename) - 1;
     while (d != c) {
       if (*d == '.') return d;
       d--;
     }
-    return c+strlen(filename);  
+    return c+Len(filename);  
   }
   return c;
 }
@@ -278,6 +313,23 @@ Swig_file_basename(const String_or_char *filename)
   return tmp;
 }
 
+/* -----------------------------------------------------------------------------
+ * Swig_file_filename()
+ *
+ * Return the file with any leading path stripped off
+ * ----------------------------------------------------------------------------- */
+char *
+Swig_file_filename(const String_or_char *filename)
+{
+  static char tmp[1024];
+  const char *delim = SWIG_FILE_DELIMETER;
+  char *c;
+
+  strcpy(tmp,Char(filename));
+  if ((c=strrchr(tmp,*delim))) return c+1;
+  else return tmp;
+}
+
 /* -----------------------------------------------------------------------------
  * Swig_file_dirname()
  *
