@@ -14,7 +14,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/parms.c,v 1.4.4.5 2001/11/05 04:28:43 beazley Exp $";
+char cvsroot_parms_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/parms.c,v 1.8 2004/01/15 22:46:07 cheetah Exp $";
 
 #include "swig.h"
 
@@ -26,7 +26,7 @@ static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/parms.c,v 1.4.4
  * Create a new parameter from datatype 'type' and name 'n'.
  * ------------------------------------------------------------------------ */
 
-Parm *NewParm(SwigType *type, String_or_char *n) {
+Parm *NewParm(SwigType *type, const String_or_char *n) {
   Parm *p = NewHash();
   
   if (type) {
@@ -42,28 +42,40 @@ Parm *NewParm(SwigType *type, String_or_char *n) {
 
 Parm *CopyParm(Parm *p) {
   SwigType *t;
-  char     *name;
-  char     *lname;
-  char     *value;
-  int       ignore;
+  String   *name;
+  String   *lname;
+  String   *value;
+  String   *ignore;
+  String   *alttype;
+  String   *byname;
 
   Parm *np = NewHash();
   t = Getattr(p,"type");
-  name = GetChar(p,"name");
-  lname = GetChar(p,"lname");
-  value = GetChar(p,"value");
-  ignore = GetInt(p,"ignore");
+  name = Getattr(p,"name");
+  lname = Getattr(p,"lname");
+  value = Getattr(p,"value");
+  ignore = Getattr(p,"ignore");
+  alttype = Getattr(p,"alttype");
+  byname = Getattr(p, "arg:byname");
 
   if (t) 
     Setattr(np,"type",Copy(t));
   if (name)
-    Setattr(np,"name",name);
+    Setattr(np,"name",Copy(name));
   if (lname)
-    Setattr(np,"lname", lname);
+    Setattr(np,"lname", Copy(lname));
   if (value)
-    Setattr(np,"value", value);
+    Setattr(np,"value", Copy(value));
   if (ignore)
-    SetInt(np,"ignore", ignore);
+    Setattr(np,"ignore", Copy(ignore));
+  if (alttype) 
+    Setattr(np,"alttype", Copy(alttype));
+  if (byname)
+    Setattr(np, "arg:byname", Copy(byname));
+      
+  Setfile(np,Getfile(p));
+  Setline(np,Getline(p));
+
   return np;
 }
 
@@ -168,6 +180,10 @@ String *ParmList_protostr(ParmList *p) {
 
   out = NewString("");
   while(p) {
+    if (Getattr(p,"hidden")) {
+      p = nextSibling(p);
+      continue;
+    }
     t = Getattr(p,"type");
     Printf(out,"%s", SwigType_str(t,0));
     p = nextSibling(p);
