@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_stype_c[] = "$Header: /cvsroot/SWIG/Source/Swig/stype.c,v 1.42 2003/11/28 19:06:35 beazley Exp $";
+char cvsroot_stype_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/stype.c,v 1.46 2004/05/27 11:29:39 marcelomatus Exp $";
 
 #include "swig.h"
 #include <ctype.h>
@@ -227,7 +227,7 @@ int SwigType_isconst(SwigType *t) {
 int SwigType_ismutable(SwigType *t) {
   int r;
   SwigType *qt = SwigType_typedef_resolve_all(t);
-  if (SwigType_isreference(qt)) {
+  if (SwigType_isreference(qt) || SwigType_isarray(qt)) {
     Delete(SwigType_pop(qt));
   }
   r = SwigType_isconst(qt);
@@ -757,6 +757,7 @@ String *SwigType_lcaststr(SwigType *s, const String_or_char *name) {
   return result;
 }
 
+/* keep old mangling since Java codes need it */
 String *SwigType_manglestr_default(SwigType *s) {
   char *c;
   String *result,*base;
