@@ -9,9 +9,13 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/typemap.c,v 1.6.2.1.2.28 2002/01/31 16:16:11 beazley Exp $";
+char cvsroot_typemap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typemap.c,v 1.33 2004/08/24 21:48:46 wsfulton Exp $";
 
 #include "swig.h"
+#include "cparse.h"
+#include <ctype.h>
+
+static void replace_embedded_typemap(String *s);
 
 /* -----------------------------------------------------------------------------
  * Typemaps are stored in a collection of nested hash tables.  Something like
@@ -38,6 +42,7 @@ static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/typemap.c,v 1.6
 
 #define MAX_SCOPE  32
 
+
 static Hash *typemaps[MAX_SCOPE];
 static int   tm_scope = 0;
 
@@ -59,11 +64,24 @@ void Swig_typemap_init() {
 static String *tmop_name(const String_or_char *op) {
   static Hash *names = 0;
   String *s;
+  /* Due to "interesting" object-identity semantics of DOH,
+     we have to make sure that we only intern strings without object
+     identity into the hash table.
+     
+     (Swig_typemap_attach_kwargs calls tmop_name several times with
+     the "same" String *op (i.e., same object identity) but differing
+     string values.)
+
+     Most other callers work around this by using char* rather than
+     String *.
+                  -- mkoeppe, Jun 17, 2003
+  */
+  const char *op_without_object_identity = Char(op);
   if (!names) names = NewHash();
-  s = Getattr(names,op);
+  s = Getattr(names, op_without_object_identity);
   if (s) return s;
   s = NewStringf("tmap:%s",op);
-  Setattr(names,op,s);
+  Setattr(names,op_without_object_identity,s);
   return s;
 }
 
@@ -224,7 +242,7 @@ Swig_typemap_copy(const String_or_char *op, ParmList *srcparms, ParmList *parms)
   SwigType *ptype;
   int ts = tm_scope;
   String *tmops, *newop;
-  if (Len(parms) != Len(srcparms)) return -1;
+  if (ParmList_len(parms) != ParmList_len(srcparms)) return -1;
 
   tmop = tmop_name(op);
   while (ts >= 0) {
@@ -248,7 +266,9 @@ Swig_typemap_copy(const String_or_char *op, ParmList *srcparms, ParmList *parms)
       p = nextSibling(p);
     }
     Delete(tmops);
+
     if (!p && tm) {
+
       /* Got some kind of match */
       Swig_typemap_register(op,parms, Getattr(tm,"code"), Getattr(tm,"locals"),Getattr(tm,"kwargs"));
       return 0;
@@ -316,7 +336,7 @@ int count_args(String *s) {
   return na;
 }
 
-void 
+int
 Swig_typemap_apply(ParmList *src, ParmList *dest) {
   String *ssig, *dsig;
   Parm   *p, *np, *lastp, *dp, *lastdp = 0;
@@ -324,6 +344,7 @@ Swig_typemap_apply(ParmList *src, ParmList *dest) {
   int     ts = tm_scope;
   SwigType *type = 0, *name;
   Hash     *tm, *sm;
+  int      match = 0;
 
   /*  Printf(stdout,"apply : %s --> %s\n", ParmList_str(src), ParmList_str(dest)); */
 
@@ -376,24 +397,25 @@ Swig_typemap_apply(ParmList *src, ParmList *dest) {
     sm = Swig_typemap_get(type,name,ts);
 
     if (sm) {
-
       /* Got a typemap.  Need to only merge attributes for methods that match our signature */
-      String *key;
-
-      for (key = Firstkey(sm); key; key = Nextkey(sm)) {
+      Iterator ki;
+      match = 1;
+      for (ki = First(sm); ki.key; ki = Next(ki)) {
 	/* Check for a signature match with the source signature */
-	if ((count_args(key) == narg) && (Strstr(key,ssig))) {
-
+	if ((count_args(ki.key) == narg) && (Strstr(ki.key,ssig))) {
+	  String *oldm;
 	  /* A typemap we have to copy */
-	  String *nkey = Copy(key);
+	  String *nkey = Copy(ki.key);
 	  Replace(nkey,ssig,dsig,DOH_REPLACE_ANY);
 
 	  /* Make sure the typemap doesn't already exist in the target map */
-	  if (!Getattr(tm,nkey)) {
+	  
+	  oldm = Getattr(tm,nkey);
+	  if (!oldm || (!Getattr(tm,"code"))) {
 	    String *code;
 	    ParmList *locals;
 	    ParmList *kwargs;
-	    Hash *sm1 = Getattr(sm,key);
+	    Hash *sm1 = ki.item;
 
 	    code = Getattr(sm1,"code");
 	    locals = Getattr(sm1,"locals");
@@ -410,6 +432,7 @@ Swig_typemap_apply(ParmList *src, ParmList *dest) {
     }
     ts--;
   }
+  return match;
 }
 
 /* -----------------------------------------------------------------------------
@@ -451,14 +474,15 @@ Swig_typemap_clear_apply(Parm *parms) {
   }
   if (tm) {
     /* Clear typemaps that match our signature */
-    String *key, *key2;
-    for (key = Firstkey(tm); key; key = Nextkey(tm)) {
-      if (Strncmp(key,"tmap:",5) == 0) {
-	int na = count_args(key);
-	if ((na == narg) && Strstr(key,tsig)) {
-	  Hash *h = Getattr(tm,key);
-	  for (key2 = Firstkey(h); key2; key2 = Nextkey(h)) {
-	    Delattr(h,key2);
+    Iterator ki, ki2;
+
+    for (ki = First(tm); ki.key; ki = Next(ki)) {
+      if (Strncmp(ki.key,"tmap:",5) == 0) {
+	int na = count_args(ki.key);
+	if ((na == narg) && Strstr(ki.key,tsig)) {
+	  Hash *h = ki.item;
+	  for (ki2 = First(h); ki2.key; ki2 = Next(ki2)) {
+	    Delattr(h,ki2.key);
 	  }
 	}
       }
@@ -488,7 +512,7 @@ static SwigType *strip_arrays(SwigType *type) {
  * ----------------------------------------------------------------------------- */
 
 Hash *
-Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *name) {
+Swig_typemap_search(const String_or_char *op, SwigType *type, const String_or_char *name, SwigType **matchtype) {
   Hash *result = 0, *tm, *tm1, *tma;
   Hash *backup = 0;
   SwigType *noarrays = 0;
@@ -496,12 +520,11 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *na
   SwigType *ctype = 0;
   int ts;
   int isarray;
-  String *cname = 0;
+  const String *cname = 0;
   SwigType *unstripped = 0;
   String   *tmop = tmop_name(op);
 
   if ((name) && Len(name)) cname = name;
-  isarray = SwigType_isarray(type);
   ts = tm_scope;
 
   while (ts >= 0) {
@@ -522,11 +545,13 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *na
 	if (result && Getattr(result,"code")) goto ret_result;
 	if (result) backup = result;
       }
-      
+      isarray = SwigType_isarray(ctype);
       if (isarray) {
-	/* If working with arrays, strip away all of the dimensions and replace with "".
+	/* If working with arrays, strip away all of the dimensions and replace with "ANY".
 	   See if that generates a match */
-	if (!noarrays) noarrays = strip_arrays(ctype);
+	if (!noarrays) {
+	  noarrays = strip_arrays(ctype);
+	}
 	tma = Getattr(typemaps[ts],noarrays);
 	if (tma && cname) {
 	  tm1 = Getattr(tma,cname);
@@ -551,36 +576,47 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *na
       if (!unstripped) {
 	unstripped = ctype;
 	ctype = SwigType_strip_qualifiers(ctype);
-	if (Cmp(ctype,unstripped)) continue;
+	if (Strcmp(ctype,unstripped) != 0) continue;    /* Types are different */
 	Delete(ctype);
 	ctype = unstripped;
 	unstripped = 0;
       }
       {
+	String *octype;
 	if (unstripped) {
-	  if (unstripped != type) Delete(unstripped);
+ 	  Delete(ctype);
+	  ctype = unstripped;
 	  unstripped = 0;
 	}
+	octype = ctype;
 	ctype = SwigType_typedef_resolve(ctype);
+	if (octype != type) Delete(octype);
       }
     }
-    
+
     /* Hmmm. Well, no match seems to be found at all. See if there is some kind of default mapping */
-    if (!primitive)
-      primitive = SwigType_default(type);
-    tm = Getattr(typemaps[ts],primitive);
-    if (tm && cname) {
-      tm1 = Getattr(tm,cname);
-      if (tm1) {
-	result = Getattr(tm1,tmop);          /* See if there is a type-name match */
+
+    primitive = SwigType_default(type);
+    while (primitive) {
+      tm = Getattr(typemaps[ts],primitive);
+      if (tm && cname) {
+	tm1 = Getattr(tm,cname);
+	if (tm1) {
+	  result = Getattr(tm1,tmop);          /* See if there is a type-name match */
+	  if (result) goto ret_result;
+	}
+      }
+      if (tm) {			/* See if there is simply a type match */
+	result = Getattr(tm,tmop);
 	if (result) goto ret_result;
       }
+      {
+	SwigType *nprim = SwigType_default(primitive);
+	Delete(primitive);
+	primitive = nprim;
+      }
     }
-    if (tm) {			/* See if there is simply a type match */
-      result = Getattr(tm,tmop);
-      if (result) goto ret_result;
-    }
-    if (ctype != type) Delete(ctype);
+    if (ctype != type) { Delete(ctype); ctype = 0; }
     ts--;         /* Hmmm. Nothing found in this scope.  Guess we'll go try another scope */
   }
   result = backup;
@@ -589,10 +625,14 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *na
   if (noarrays) Delete(noarrays);
   if (primitive) Delete(primitive);
   if ((unstripped) && (unstripped != type)) Delete(unstripped);
+  if (matchtype) {
+    *matchtype = Copy(ctype);
+  }
   if (type != ctype) Delete(ctype);
   return result;
 }
 
+
 /* -----------------------------------------------------------------------------
  * Swig_typemap_search_multi()
  *
@@ -602,6 +642,7 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *na
 Hash *
 Swig_typemap_search_multi(const String_or_char *op, ParmList *parms, int *nmatch) {
   SwigType *type;
+  SwigType *mtype = 0;
   String   *name;
   String   *newop;
   Hash     *tm, *tm1;
@@ -614,8 +655,12 @@ Swig_typemap_search_multi(const String_or_char *op, ParmList *parms, int *nmatch
   name = Getattr(parms,"name");
 
   /* Try to find a match on the first type */
-  tm = Swig_typemap_search(op, type, name);
+  tm = Swig_typemap_search(op, type, name, &mtype);
   if (tm) {
+    if (mtype && SwigType_isarray(mtype)) {
+      Setattr(parms,"tmap:match", mtype);
+    }
+    Delete(mtype);
     newop = NewStringf("%s-%s+%s:", op, type,name);
     tm1 = Swig_typemap_search_multi(newop, nextSibling(parms), nmatch);
     if (tm1) tm = tm1;
@@ -638,7 +683,7 @@ Swig_typemap_search_multi(const String_or_char *op, ParmList *parms, int *nmatch
  * ----------------------------------------------------------------------------- */
 
 static
-void replace_local_types(ParmList *p, String *name, String *rep) {
+void replace_local_types(ParmList *p, const String *name, const String *rep) {
   SwigType *t;
   while (p) {
     t = Getattr(p,"type");
@@ -647,11 +692,26 @@ void replace_local_types(ParmList *p, String *name, String *rep) {
   }
 }
 
+static
+int check_locals(ParmList *p, const char *s) {
+  while (p) {
+    char *c = GetChar(p,"type");
+    if (strstr(c,s)) return 1;
+    p = nextSibling(p);
+  }
+  return 0;
+}
+
 static
 void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *pname, String *lname, int index) 
 {
   char var[512];
   char *varname;
+  SwigType *ftype;
+
+  Replaceall(s,"$typemap","$TYPEMAP");
+
+  ftype = SwigType_typedef_resolve_all(type);
 
   if (!pname) pname = lname;
   {
@@ -669,11 +729,15 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
   varname = &var[strlen(var)];
     
   /* If the original datatype was an array. We're going to go through and substitute
-     it's array dimensions */
+     its array dimensions */
     
-  if (SwigType_isarray(type)) {
-    int  ndim = SwigType_array_ndim(type);
+  if (SwigType_isarray(type) || SwigType_isarray(ftype)) {
+    String *size;
+    int ndim;
     int i;
+    if (!SwigType_isarray(type)) type = ftype;
+    ndim = SwigType_array_ndim(type);
+    size = NewString("");
     for (i = 0; i < ndim; i++) {
       String *dim = SwigType_array_getdim(type,i);
       if (index == 1) {
@@ -685,8 +749,14 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
       sprintf(varname,"dim%d",i);
       Replace(s,var,dim,DOH_REPLACE_ANY);
       replace_local_types(locals,var,dim);	
+      if (Len(size)) Putc('*',size);
+      Append(size,dim);
       Delete(dim);
     }
+    sprintf(varname,"size");
+    Replace(s,var,size,DOH_REPLACE_ANY);
+    replace_local_types(locals,var,size);	    
+    Delete(size);
   }
 
   /* Parameter name substitution */
@@ -700,14 +770,14 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
   {
     SwigType *star_type, *amp_type, *base_type;
     SwigType *ltype, *star_ltype, *amp_ltype;
-    String *mangle, *star_mangle, *amp_mangle, *base_mangle;
+    String *mangle, *star_mangle, *amp_mangle, *base_mangle, *base_name;
     String *descriptor, *star_descriptor, *amp_descriptor;
     String *ts;
     char   *sc;
 
     sc = Char(s);
 
-    if (strstr(sc,"type")) {
+    if (strstr(sc,"type") || check_locals(locals,"type")) {
       /* Given type : $type */
       ts = SwigType_str(type,0);
       if (index == 1) {
@@ -720,7 +790,7 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
       Delete(ts);
       sc = Char(s);
     }
-    if (strstr(sc,"ltype")) {
+    if (strstr(sc,"ltype") || check_locals(locals,"ltype")) {
       /* Local type:  $ltype */
       ltype = SwigType_ltype(type);
       ts = SwigType_str(ltype,0);
@@ -745,6 +815,7 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
       Replace(s,var,mangle,DOH_REPLACE_ANY);
     
       descriptor = NewStringf("SWIGTYPE%s", mangle);
+
       if (index == 1)
 	if (Replace(s, "$descriptor", descriptor, DOH_REPLACE_ANY))
 	  SwigType_remember(type);
@@ -763,19 +834,30 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
           $*n_ltype
     */
 
-    if (SwigType_ispointer(type)) {
-      star_type = Copy(type);
-      SwigType_del_pointer(star_type);
-      ts = SwigType_str(star_type,0);
-      if (index == 1) {
-	Replace(s, "$*type", ts, DOH_REPLACE_ANY);
-	replace_local_types(locals,"$*type",star_type);
+    if (SwigType_ispointer(ftype) || (SwigType_isarray(ftype)) || (SwigType_isreference(ftype))) {
+      if (!(SwigType_isarray(type) || SwigType_ispointer(type) || SwigType_isreference(type))) {
+	star_type = Copy(ftype);
+      } else {
+	star_type = Copy(type);
+      }
+      if (!SwigType_isreference(star_type)) {
+	if (SwigType_isarray(star_type)) {
+	  SwigType_del_element(star_type);
+	} else {
+	  SwigType_del_pointer(star_type);
+	}
+	ts = SwigType_str(star_type,0);
+	if (index == 1) {
+	  Replace(s, "$*type", ts, DOH_REPLACE_ANY);
+	  replace_local_types(locals,"$*type",star_type);
+	}
+	sprintf(varname,"$*%d_type",index);
+	Replace(s,varname,ts,DOH_REPLACE_ANY);
+	replace_local_types(locals,varname,star_type);
+	Delete(ts);
+      } else {
+	SwigType_del_element(star_type);
       }
-      sprintf(varname,"$*%d_type",index);
-      Replace(s,varname,ts,DOH_REPLACE_ANY);
-      replace_local_types(locals,varname,star_type);
-      Delete(ts);
-      
       star_ltype = SwigType_ltype(star_type);
       ts = SwigType_str(star_ltype,0);
       if (index == 1) {
@@ -825,7 +907,8 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
     replace_local_types(locals,varname,amp_type);
     Delete(ts);
     
-    amp_ltype = SwigType_ltype(amp_type);
+    amp_ltype = SwigType_ltype(type);
+    SwigType_add_pointer(amp_ltype);
     ts = SwigType_str(amp_ltype,0);
     
     if (index == 1) {
@@ -858,14 +941,23 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
     Delete(amp_type);
 
     /* Base type */
-    base_type = SwigType_base(type);
+    if (SwigType_isarray(type)) {
+      SwigType *bt = Copy(type);
+      Delete(SwigType_pop_arrays(bt));
+      base_type = SwigType_str(bt,0);
+      Delete(bt);
+    } else {
+      base_type = SwigType_base(type);
+    }
+
+    base_name = SwigType_namestr(base_type);
     if (index == 1) {
-      Replace(s,"$basetype", base_type, DOH_REPLACE_ANY);
-      replace_local_types(locals,"$basetype", base_type);
+      Replace(s,"$basetype", base_name, DOH_REPLACE_ANY);
+      replace_local_types(locals,"$basetype", base_name);
     }
     strcpy(varname,"basetype");
     Replace(s,var,base_type,DOH_REPLACE_ANY);
-    replace_local_types(locals,var,base_type);
+    replace_local_types(locals,var,base_name);
     
     base_mangle = SwigType_manglestr(base_type);
     if (index == 1)
@@ -874,6 +966,7 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
     Replace(s,var,base_mangle,DOH_REPLACE_ANY);
     Delete(base_mangle);
     Delete(base_type);
+    Delete(base_name);
   }
   
   /* Replace any $n. with (&n)-> */
@@ -887,6 +980,7 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
   /* Replace the bare $n variable */
   sprintf(var,"$%d",index);
   Replace(s,var,lname,DOH_REPLACE_ANY);
+  Delete(ftype);
 }
 
 /* ------------------------------------------------------------------------
@@ -899,32 +993,44 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
 static void typemap_locals(DOHString *s, ParmList *l, Wrapper *f, int argnum) {
   Parm *p;
   char *new_name;
-  SwigType *pbase = 0;
 
   p = l;
   while (p) {
     SwigType *pt = Getattr(p,"type");
     String   *pn = Getattr(p,"name");
+    String   *value = Getattr(p,"value");
     if (pn) {
       if (Len(pn) > 0) {
-	DOHString *str;
+	String *str;
+	int     isglobal = 0;
 
 	str = NewString("");
 
+	if (Strncmp(pn,"_global_",8) == 0) {
+	    isglobal = 1;
+	}
+
 	/* If the user gave us $type as the name of the local variable, we'll use
 	   the passed datatype instead */
 
-	if (argnum >= 0) {
+	if ((argnum >= 0) && (!isglobal)) {
 	  Printf(str,"%s%d",pn,argnum);
 	} else {
 	  Printf(str,"%s",pn);
 	}
-	/* Substitute parameter names */
-	/*	Replace(str,"$arg",pname, DOH_REPLACE_ANY);    /* This is deprecated (or should be) */
-	new_name = Wrapper_new_localv(f,str, SwigType_str(pt,str), 0);
-	/* Substitute  */
-	Replace(s,pn,new_name,DOH_REPLACE_ID);
-	Delete(pbase);
+	if (isglobal && Wrapper_check_local(f,str)) {
+	    p = nextSibling(p);
+	    continue;
+	}
+	if (value) {
+	    new_name = Wrapper_new_localv(f,str, SwigType_str(pt,str), "=", value, NIL);
+	} else {
+	    new_name = Wrapper_new_localv(f,str, SwigType_str(pt,str), NIL);
+	}
+	if (!isglobal) {
+	    /* Substitute  */
+	    Replace(s,pn,new_name,DOH_REPLACE_ID | DOH_REPLACE_NOQUOTE);
+	}
       }
     }
     p = nextSibling(p);
@@ -943,8 +1049,9 @@ String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_
 {
   Hash   *tm;
   String *s = 0;
+  SwigType *mtype = 0;
   ParmList *locals;
-  tm = Swig_typemap_search(op,type,pname);
+  tm = Swig_typemap_search(op,type,pname,&mtype);
   if (!tm) return 0;
 
   s = Getattr(tm,"code");
@@ -959,22 +1066,33 @@ String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_
   if (locals) locals = CopyParmList(locals);
 
   /* This is wrong.  It replaces locals in place.   Need to fix this */
-  typemap_replace_vars(s,locals,type,pname,lname,1);
+  if (mtype && SwigType_isarray(mtype)) {
+    typemap_replace_vars(s,locals,mtype,pname,lname,1);
+  } else {
+    typemap_replace_vars(s,locals,type,pname,lname,1);
+  }
 
   if (locals && f) {
     typemap_locals(s,locals,f,-1);
   }
-   
+
+  replace_embedded_typemap(s);   
+
   /* Now perform character replacements */
   Replace(s,"$source",source,DOH_REPLACE_ANY);
   Replace(s,"$target",target,DOH_REPLACE_ANY);
-  {
+
+  /*  {
     String *tmname = Getattr(tm,"typemap");
     if (tmname) Replace(s,"$typemap",tmname, DOH_REPLACE_ANY);
   }
+  */
+
   Replace(s,"$parmname",pname, DOH_REPLACE_ANY);
   /*  Replace(s,"$name",pname,DOH_REPLACE_ANY); */
+
   Delete(locals);
+  Delete(mtype);
   return s;
 }
 
@@ -984,44 +1102,74 @@ String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_
  * Attach one or more typemaps to a node
  * ----------------------------------------------------------------------------- */
 
+Node *
+Swig_cparse_template_locate(String *name, Parm *tparms);
+
 String *Swig_typemap_lookup_new(const String_or_char *op, Node *node, const String_or_char *lname, Wrapper *f)
 {
   SwigType *type;
+  SwigType *mtype = 0;
   String   *pname;
   Hash     *tm;
   String   *s = 0;
+  String   *sdef = 0;
   ParmList *locals;
   ParmList *kw;
   char     temp[256];
   String   *symname;
+  String   *cname = 0;
+  String   *clname = 0;
+
+  /* special case, we need to check for 'ref' call 
+     and set the defaul code 'sdef' */
+  if (Cmp(op,"newfree") == 0) { 
+    sdef = Swig_ref_call(node, lname);
+  }
 
   type  = Getattr(node,"type");
-  if (!type) return 0;
+  if (!type) return sdef;
 
   pname = Getattr(node,"name");
-  tm = Swig_typemap_search(op,type,pname);
-  if (!tm) return 0;
+  tm = Swig_typemap_search(op,type,pname,&mtype);
+  if (!tm) return sdef;
 
   s = Getattr(tm,"code");
-  if (!s) return 0;
+  if (!s) return sdef;
 
   /* Empty typemap. No match */
-  if (Cmp(s,"pass") == 0) return 0;
+  if (Cmp(s,"pass") == 0) return sdef;
 
   s = Copy(s);             /* Make a local copy of the typemap code */
 
   locals = Getattr(tm,"locals");
   if (locals) locals = CopyParmList(locals);
 
-  typemap_replace_vars(s,locals,type,pname,(char *) lname,1);
+  if (pname) {
+    if (SwigType_istemplate(pname)) {
+      cname = SwigType_namestr(pname);
+      pname = cname;
+    }
+  }
+  if (SwigType_istemplate((char*)lname)) {
+    clname = SwigType_namestr((char *)lname);
+    lname = clname;
+  }
+
+  if (mtype && SwigType_isarray(mtype)) {
+    typemap_replace_vars(s,locals,mtype,pname,(char *) lname,1);
+  } else {
+    typemap_replace_vars(s,locals,type,pname,(char *) lname,1);
+  }
 
   if (locals && f) {
     typemap_locals(s,locals,f,-1);
   }
-  {
+  replace_embedded_typemap(s);
+  /*  {
     String *tmname = Getattr(tm,"typemap");
     if (tmname) Replace(s,"$typemap",tmname, DOH_REPLACE_ANY);
-  }
+    }*/
+
   Replace(s,"$name",pname,DOH_REPLACE_ANY);
   
   symname = Getattr(node,"sym:name");
@@ -1036,29 +1184,123 @@ String *Swig_typemap_lookup_new(const String_or_char *op, Node *node, const Stri
     Delete(locals);  
   }
 
+  if (checkAttribute(tm,"type","SWIGTYPE")) {
+    sprintf(temp,"%s:SWIGTYPE", Char(op));
+    Setattr(node,tmop_name(temp),"1");
+  }
+
   /* Attach kwargs */
   kw = Getattr(tm,"kwargs");
   while (kw) {
+    String *value = Copy(Getattr(kw,"value"));
+    String *type = Getattr(kw,"type");
+    if (type) {
+      SwigType *rtype = SwigType_typedef_resolve_all(type);
+      String *mangle = Swig_string_mangle(rtype);
+      Printf(value,"%s",mangle);
+      Delete(mangle);
+      Delete(rtype);
+    }
     sprintf(temp,"%s:%s",Char(op),Char(Getattr(kw,"name")));
-    Setattr(node,tmop_name(temp), Copy(Getattr(kw,"value")));
+    Setattr(node,tmop_name(temp), value);
     kw = nextSibling(kw);
   }
+
+  /* Look for warnings */
   {
     String *w;
     sprintf(temp,"%s:warning", Char(op));
     w = Getattr(node,tmop_name(temp));
     if (w) {
-      Printf(stderr,"%s:%d. Typemap warning. %s\n", Getfile(node), Getline(node), w);
+      Swig_warning(0,Getfile(node),Getline(node),"%s\n", w);
     }
   }
 
+  /* Look for code fragments */
+  {
+    String *f;
+    sprintf(temp,"%s:fragment", Char(op));
+    f = Getattr(node,tmop_name(temp));
+    if (f) {
+      Swig_fragment_emit(f);
+    }
+  }
+    
+  if (cname) Delete(cname);
+  if (clname) Delete(clname);
+  if (mtype) Delete(mtype);
+  if (sdef) { /* put 'ref' and 'newfree' codes together */
+    String *p = NewStringf("%s\n%s", sdef, s);
+    Delete(s);
+    Delete(sdef);
+    s = p;
+  }
   return s;
 }
 
+/* -----------------------------------------------------------------------------
+ * Swig_typemap_attach_kwargs()
+ *
+ * If this hash (tm) contains a linked list of parameters under its "kwargs"
+ * attribute, add keys for each of those named keyword arguments to this
+ * parameter for later use.
+ * For example, attach the typemap attributes to p:
+ * %typemap(in, foo="xyz") ...
+ * A new attribute called "tmap:in:foo" with value "xyz" is attached to p.
+ * ----------------------------------------------------------------------------- */
+
+void
+Swig_typemap_attach_kwargs(Hash *tm, const String_or_char *op, Parm *p) {
+  String *temp = NewString("");
+  Parm *kw = Getattr(tm,"kwargs");
+  while (kw) {
+    String *value = Copy(Getattr(kw,"value"));
+    String *type = Getattr(kw,"type");
+    if (type) {
+      Hash *v = NewHash();
+      Setattr(v,"value",value);
+      Setattr(v,"type",type);
+      value = v;
+    }
+    Clear(temp);
+    Printf(temp,"%s:%s",op,Getattr(kw,"name"));
+    Setattr(p,tmop_name(temp),value);
+    kw = nextSibling(kw);
+  }
+  Delete(temp);
+}
+
+/* -----------------------------------------------------------------------------
+ * Swig_typemap_warn()
+ *
+ * If any warning message is attached to this parameter's "tmap:op:warning"
+ * attribute, print that warning message.
+ * ----------------------------------------------------------------------------- */
+
+static void
+Swig_typemap_warn(const String_or_char *op, Parm *p) {
+  String *temp = NewStringf("%s:warning",op);
+  String *w = Getattr(p,tmop_name(temp));
+  Delete(temp);
+  if (w) {
+    Swig_warning(0,Getfile(p),Getline(p),"%s\n",w);
+  }
+}
+
+static void
+Swig_typemap_emit_code_fragments(const String_or_char *op, Parm *p) {
+  String *temp = NewStringf("%s:fragment",op);
+  String *f = Getattr(p,tmop_name(temp));
+  if (f) {
+    Swig_fragment_emit(f);
+  }
+  Delete(temp);
+}
+
 /* -----------------------------------------------------------------------------
  * Swig_typemap_attach_parms()
  *
- * Given a parmeter list, this function attaches all of the typemaps for a
+ * Given a parameter list, this function attaches all of the typemaps for a
  * given typemap type
  * ----------------------------------------------------------------------------- */
 
@@ -1072,7 +1314,6 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
   ParmList *locals;
   int   argnum = 0;
   char  temp[256];
-  Parm  *kw;
 
   p = parms;
   while (p) {
@@ -1103,12 +1344,24 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
       SwigType *type;
       String   *pname;
       String   *lname;
+      SwigType *mtype;
 
       type = Getattr(p,"type");
       pname = Getattr(p,"name");
       lname = Getattr(p,"lname");
+      mtype = Getattr(p,"tmap:match");
 
-      typemap_replace_vars(s,locals, type,pname,lname,i+1);
+      if (mtype) {
+	typemap_replace_vars(s,locals, mtype,pname,lname,i+1);
+	Delattr(p,"tmap:match");
+      } else {
+	typemap_replace_vars(s,locals, type,pname,lname,i+1);
+      }
+
+      if (checkAttribute(tm,"type","SWIGTYPE")) {
+	sprintf(temp,"%s:SWIGTYPE", Char(op));
+	Setattr(p,tmop_name(temp),"1");
+      }
       p = nextSibling(p);
     }
     
@@ -1116,6 +1369,8 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
       typemap_locals(s,locals,f,argnum);
     }
 
+    replace_embedded_typemap(s);   
+
     /* Replace the argument number */
     sprintf(temp,"%d",argnum);
     Replace(s,"$argnum",temp, DOH_REPLACE_ANY);
@@ -1134,20 +1389,209 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
     Setattr(firstp,tmop_name(temp),p);
 
     /* Attach kwargs */
-    kw = Getattr(tm,"kwargs");
-    while (kw) {
-      sprintf(temp,"%s:%s",Char(op),Char(Getattr(kw,"name")));
-      Setattr(firstp,tmop_name(temp), Copy(Getattr(kw,"value")));
-      kw = nextSibling(kw);
-    }
-    {
-      String *w;
-      sprintf(temp,"%s:warning", Char(op));
-      w = Getattr(firstp,tmop_name(temp));
-      if (w) {
-	Printf(stderr,"%s:%d. Typemap warning. %s\n", Getfile(firstp), Getline(firstp), w);
+    Swig_typemap_attach_kwargs(tm,op,firstp);
+    
+    /* Print warnings, if any */
+    Swig_typemap_warn(op,firstp);
+
+    /* Look for code fragments */
+    Swig_typemap_emit_code_fragments(op,firstp);
+  }
+}
+
+/* -----------------------------------------------------------------------------
+ * split_embedded()
+ *
+ * This function replaces the special variable $typemap(....) with typemap
+ * code.  The general form of $typemap is as follows:
+ *
+ *   $TYPEMAP(method, $var1=value, $var2=value, $var3=value,...)
+ *
+ * For example:
+ *
+ *   $TYPEMAP(in, $1=int x, $input=y, ...)
+ *
+ * ----------------------------------------------------------------------------- */
+
+/* Splits the arguments of an embedded typemap */
+static List *split_embedded(String *s) {
+  List *args = 0;
+  char *c,*start;
+  int  level=0;
+  int  leading = 1;
+  args = NewList();
+  
+  c = Strstr(s,"(");
+  c++;
+
+  start = c;
+  while (*c) {
+    if (*c == '\"') {
+      c++;
+      while (*c) {
+	if (*c == '\\') {
+	  c++;
+	} else {
+	  if (*c == '\"') break;
+	}
+	c++;
       }
     }
+    if ((level == 0) && ((*c == ',') || (*c == ')'))) {
+      String *tmp = NewStringWithSize(start,c-start);
+      Append(args,tmp);
+      Delete(tmp);
+      start = c+1;
+      leading = 1;
+      if (*c == ')') break;
+      c++;
+      continue;
+    }
+    if (*c == '(') level++;
+    if (*c == ')') level--;
+    if (isspace((int)*c) && leading) start++; 
+    if (!isspace((int)*c)) leading = 0;
+    c++;
+  }
+  return args;
+}
+
+static void split_var(String *s, String **name, String **value) {
+  char *eq;
+  char *c;
+
+  eq = Strstr(s,"=");
+  if (!eq) {
+    *name = 0;
+    *value = 0;
+    return;
+  }
+  c = Char(s);
+  *name = NewStringWithSize(c,eq-c);
+  
+  /* Look for $n variables */
+  if (isdigit((int)*(c))) {
+    /* Parse the value as a type */
+    String *v;
+    Parm *p;
+    v = NewString(eq+1);
+    p = Swig_cparse_parm(v);
+    Delete(v);
+    *value = p;
+  } else {
+    *value = NewString(eq+1);
+  }
+}
+
+static void replace_embedded_typemap(String *s) {
+  while (Strstr(s,"$TYPEMAP(")) {
+
+    /* Gather the argument */
+    char *start, *end=0,*c;
+    int  level = 0;
+    String *tmp;
+    start = Strstr(s,"$TYPEMAP(");
+    c = start;
+    while (*c) {
+      if (*c == '(') level++;
+      if (*c == ')') {
+	level--;
+	if (level == 0) {
+	  end = c+1;
+	  break;
+	}
+      }
+      c++;
+    }
+    if (end) {
+      tmp = NewStringWithSize(start,(end-start));
+    } else {
+      tmp = 0;
+    }
+
+    /* Got a substitution. Split it apart into pieces */
+    if (tmp) {
+      List   *l;
+      Hash   *vars;
+      String *method;
+      int i;
+
+      l = split_embedded(tmp);
+      vars = NewHash();
+      for (i = 1; i < Len(l); i++) {
+	String *n, *v;
+	split_var(Getitem(l,i),&n,&v);
+	if (n && v) {
+	  Insert(n,0,"$");
+	  Setattr(vars,n,v);
+	}
+      }
+
+      method = Getitem(l,0);
+      /* Generate the parameter list for matching typemaps */
+
+      {
+	Parm *p = 0;
+	Parm *first = 0;
+	char  temp[32];
+	int   n = 1;
+	while (1) {
+	  Hash *v;
+	  sprintf(temp,"$%d",n);
+	  v = Getattr(vars,temp);
+	  if (v) {
+	    if (p) {
+	      set_nextSibling(p,v);
+	      set_previousSibling(v,p);
+	    }
+	    p = v;
+	    Setattr(p,"lname",Getattr(p,"name"));
+	    if (Getattr(p,"value")) {
+	      Setattr(p,"name",Getattr(p,"value"));
+	    }
+	    if (!first) first = p;
+	    DohIncref(p);
+	    Delattr(vars,temp);
+	  } else {
+	    break;
+	  }
+	  n++;
+	}
+	/* Perform a typemap search */
+	if (first) {
+	  Swig_typemap_attach_parms(method,first,0);
+	  {
+	    String *tm;
+	    int     match = 0;
+	    char attr[64];
+	    sprintf(attr,"tmap:%s",Char(method));
+	    
+	    /* Look for the typemap code */
+	    tm = Getattr(first,attr);
+	    if (tm) {
+	      sprintf(attr,"tmap:%s:next",Char(method));
+	      if (!Getattr(first,attr)) {
+		/* Should be no more matches.  Hack??? */
+		/* Replace all of the remaining variables */
+		Iterator ki;
+		for (ki = First(vars); ki.key; ki = Next(ki)) {
+		  Replace(tm,ki.key,ki.item, DOH_REPLACE_ANY);
+		}
+		/* Do the replacement */
+		Replace(s,tmp,tm, DOH_REPLACE_ANY);
+		Delete(tm);
+		Delete(vars);
+		match = 1;
+	      }
+	    }
+	    if (!match) {
+	      Swig_error(Getfile(s),Getline(s),"No typemap found for %s\n", tmp);
+	    }
+	  }
+	}
+      }
+      Replace(s,tmp,"<embedded typemap>", DOH_REPLACE_ANY);
+    }
   }
 }
 
@@ -1167,36 +1611,5 @@ void Swig_typemap_debug() {
   }
   Printf(stdout,"-----------------------------------------------------------------------------\n");
 }
- 
 
-/* -----------------------------------------------------------------------------
- * %except directive support.
- *
- * These functions technically don't really have anything to do with typemaps
- * except that they have the same scoping rules.  Therefore, it's easy enough to 
- * just use the hash table structure of the typemap code.
- * ----------------------------------------------------------------------------- */
 
-void Swig_except_register(String_or_char *code) {
-  String *s = NewString(code);
-  Setattr(typemaps[tm_scope],"*except*",s);
-  Delete(s);
-}
-
-String *Swig_except_lookup() {
-  String *s;
-  int ts = tm_scope;
-  while (ts >= 0) {
-    s = Getattr(typemaps[tm_scope],"*except*");
-    if (s) {
-      s = Copy(s);
-      return s;
-    }
-    ts--;
-  }
-  return 0;
-}
-
-void Swig_except_clear() {
-  Delattr(typemaps[tm_scope],"*except*");
-}
