@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typeobj_c[] = "$Header: /cvsroot/SWIG/Source/Swig/typeobj.c,v 1.4 2003/09/11 20:26:57 beazley Exp $";
+char cvsroot_typeobj_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typeobj.c,v 1.7 2004/08/29 22:02:31 wsfulton Exp $";
 
 #include "swig.h"
 #include <ctype.h>
@@ -363,7 +363,8 @@ SwigType_add_reference(SwigType *t) {
 void
 SwigType_del_reference(SwigType *t) {
   char *c = Char(t);
-  assert(strncmp(c,"r.",2) == 0);
+  int check = strncmp(c,"r.",2);
+  assert(check == 0);
   Delslice(t,0,2);
 }
 
@@ -393,7 +394,7 @@ SwigType_isreference(SwigType *t) {
  * ----------------------------------------------------------------------------- */
 
 void
-SwigType_add_qualifier(SwigType *t, String *qual) {
+SwigType_add_qualifier(SwigType *t, const String_or_char *qual) {
   char temp[256], newq[256];
   int  sz, added = 0;
   char *q, *cqual;
@@ -452,8 +453,9 @@ SwigType_add_qualifier(SwigType *t, String *qual) {
 void
 SwigType_del_qualifier(SwigType *t) {
   char *c = Char(t);
-  assert(strncmp(c,"q(",2) == 0);
-  Delslice(t,0,element_size(t));
+  int check = strncmp(c,"q(",2);
+  assert(check == 0);
+  Delslice(t,0,element_size(Char(t)));
 }
 
 int 
@@ -478,7 +480,7 @@ SwigType_isqualifier(SwigType *t) {
  * ----------------------------------------------------------------------------- */
 
 void
-SwigType_add_memberpointer(SwigType *t, String_or_char *name) {
+SwigType_add_memberpointer(SwigType *t, const String_or_char *name) {
   String *temp = NewStringf("m(%s).", name);
   Insert(t,0,temp);
   Delete(temp);
@@ -487,7 +489,8 @@ SwigType_add_memberpointer(SwigType *t, String_or_char *name) {
 void
 SwigType_del_memberpointer(SwigType *t) {
   char *c = Char(t);
-  assert(strncmp(c,"m(",2) == 0);
+  int check = strncmp(c,"m(",2);
+  assert(check == 0);
   Delslice(t,0,element_size(c));
 }
 
@@ -519,7 +522,7 @@ SwigType_ismemberpointer(SwigType *t) {
  * ----------------------------------------------------------------------------- */
 
 void
-SwigType_add_array(SwigType *t, String *size) {
+SwigType_add_array(SwigType *t, const String_or_char *size) {
   char temp[512];
   strcpy(temp,"a(");
   strcat(temp,Char(size));
@@ -530,7 +533,8 @@ SwigType_add_array(SwigType *t, String *size) {
 void
 SwigType_del_array(SwigType *t) {
   char *c = Char(t);
-  assert(strncmp(c,"a(",2) == 0);
+  int check = strncmp(c,"a(",2);
+  assert(check == 0);
   Delslice(t,0,element_size(c));
 }
 
@@ -586,7 +590,7 @@ SwigType_array_getdim(SwigType *t, int n) {
 }
 
 /* Replace nth array dimension */
-void SwigType_array_setdim(SwigType *t, int n, String_or_char *rep) {
+void SwigType_array_setdim(SwigType *t, int n, const String_or_char *rep) {
   String *result = 0;
   char temp;
   char *start;
